# -*- coding: utf-8 -*-
"""
A module containing `Graphics View` for NodeEditor
"""
from PyQt5.QtWidgets import QGraphicsView, QApplication
from PyQt5.QtCore import *
from PyQt5.QtGui import *

from NodeEditor.nodeeditor.node_graphics_socket import QDMGraphicsSocket
from NodeEditor.nodeeditor.node_graphics_edge import QDMGraphicsEdge
from NodeEditor.nodeeditor.node_edge_dragging import EdgeDragging
from NodeEditor.nodeeditor.node_graphics_cutline import QDMCutLine
from NodeEditor.nodeeditor.utils import dumpException


MODE_NOOP = 1               #: Mode representing ready state
MODE_EDGE_DRAG = 2          #: Mode representing when we drag edge state
MODE_EDGE_CUT = 3           #: Mode representing when we draw a cutting edge

#: Distance when click on socket to enable `Drag Edge`
EDGE_DRAG_START_TRHESHOLD = 50


DEBUG = False
DEBUG_MMB_SCENE_ITEMS = False
DEBUG_MMB_LAST_SELECTIONS = False


class QDMGraphicsView(QGraphicsView):
    """Class representing NodeEditor's `Graphics View`"""
    #: pyqtSignal emitted when cursor position on the `Scene` has changed
    scenePosChanged = pyqtSignal(int, int)

    def __init__(self, grScene:'QDMGraphicsScene', parent:'QWidget'=None):
        """
        :param grScene: reference to the :class:`~nodeeditor.node_graphics_scene.QDMGraphicsScene`
        :type grScene: :class:`~nodeeditor.node_graphics_scene.QDMGraphicsScene`
        :param parent: parent widget
        :type parent: ``QWidget``

        :Instance Attributes:

        - **grScene** - reference to the :class:`~nodeeditor.node_graphics_scene.QDMGraphicsScene`
        - **mode** - state of the `Graphics View`
        - **zoomInFactor**- ``float`` - zoom step scaling, default 1.25
        - **zoomClamp** - ``bool`` - do we clamp zooming or is it infinite?
        - **zoom** - current zoom step
        - **zoomStep** - ``int`` - the relative zoom step when zooming in/out
        - **zoomRange** - ``[min, max]``

        """
        super().__init__(parent)
        self.grScene = grScene

        self.initUI()

        self.setScene(self.grScene)

        self.mode = MODE_NOOP
        self.editingFlag = False
        self.rubberBandDraggingRectangle = False

        # edge dragging
        self.dragging = EdgeDragging(self)

        # cutline
        self.cutline = QDMCutLine()
        self.grScene.addItem(self.cutline)

        self.last_scene_mouse_position = QPoint(0,0)
        self.zoomInFactor = 1.25
        self.zoomClamp = True
        self.zoom = 10
        self.zoomStep = 1
        self.zoomRange = [0, 10]

        # listeners
        self._drag_enter_listeners = []
        self._drop_listeners = []


    def initUI(self):
        """Set up this ``QGraphicsView``"""
        self.setRenderHints(QPainter.Antialiasing | QPainter.HighQualityAntialiasing | QPainter.TextAntialiasing | QPainter.SmoothPixmapTransform)

        self.setViewportUpdateMode(QGraphicsView.FullViewportUpdate)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
        self.setDragMode(QGraphicsView.RubberBandDrag)

        # enable dropping
        self.setAcceptDrops(True)

    def resetMode(self):
        """Helper function to re-set the grView's State Machine state to the default"""
        self.mode = MODE_NOOP

    def dragEnterEvent(self, event:QDragEnterEvent):
        """Trigger our registered `Drag Enter` events"""
        for callback in self._drag_enter_listeners: callback(event)

    def dropEvent(self, event:QDropEvent):
        """Trigger our registered `Drop` events"""
        for callback in self._drop_listeners: callback(event)

    def addDragEnterListener(self, callback:'function'):
        """
        Register callback for `Drag Enter` event

        :param callback: callback function
        """
        self._drag_enter_listeners.append(callback)

    def addDropListener(self, callback:'function'):
        """
        Register callback for `Drop` event

        :param callback: callback function
        """
        self._drop_listeners.append(callback)

    def mousePressEvent(self, event:QMouseEvent):
        """Dispatch Qt's mousePress event to corresponding function below"""
        if event.button() == Qt.MiddleButton:
            self.middleMouseButtonPress(event)
        elif event.button() == Qt.LeftButton:
            self.leftMouseButtonPress(event)
        elif event.button() == Qt.RightButton:
            self.rightMouseButtonPress(event)
        else:
            super().mousePressEvent(event)

    def mouseReleaseEvent(self, event:QMouseEvent):
        """Dispatch Qt's mouseRelease event to corresponding function below"""
        if event.button() == Qt.MiddleButton:
            self.middleMouseButtonRelease(event)
        elif event.button() == Qt.LeftButton:
            self.leftMouseButtonRelease(event)
        elif event.button() == Qt.RightButton:
            self.rightMouseButtonRelease(event)
        else:
            super().mouseReleaseEvent(event)


    def middleMouseButtonPress(self, event:QMouseEvent):
        """When Middle mouse button was pressed"""

        item = self.getItemAtClick(event)

        # debug print out
        if DEBUG_MMB_SCENE_ITEMS:
            if isinstance(item, QDMGraphicsEdge):
                print("MMB DEBUG:", item.edge, "\n\t", item.edge.grEdge if item.edge.grEdge is not None else None)
                return

            if isinstance(item, QDMGraphicsSocket):
                print("MMB DEBUG:", item.socket, "socket_type:", item.socket.socket_type,
                      "has edges:", "no" if item.socket.edges == [] else "")
                if item.socket.edges:
                    for edge in item.socket.edges: print("\t", edge)
                return

        if DEBUG_MMB_SCENE_ITEMS and (item is None):
            print("SCENE:")
            print("  Nodes:")
            for node in self.grScene.scene.nodes: print("\t", node)
            print("  Edges:")
            for edge in self.grScene.scene.edges: print("\t", edge, "\n\t\tgrEdge:", edge.grEdge if edge.grEdge is not None else None)

            if event.modifiers() & Qt.CTRL:
                print("  Graphic Items in GraphicScene:")
                for item in self.grScene.items():
                    print('    ', item)

        if DEBUG_MMB_LAST_SELECTIONS and event.modifiers() & Qt.SHIFT:
            print("scene _last_selected_items:", self.grScene.scene._last_selected_items)
            return

        # faking events for enable MMB dragging the scene
        releaseEvent = QMouseEvent(QEvent.MouseButtonRelease, event.localPos(), event.screenPos(),
                                   Qt.LeftButton, Qt.NoButton, event.modifiers())
        super().mouseReleaseEvent(releaseEvent)
        self.setDragMode(QGraphicsView.ScrollHandDrag)
        fakeEvent = QMouseEvent(event.type(), event.localPos(), event.screenPos(),
                                Qt.LeftButton, event.buttons() | Qt.LeftButton, event.modifiers())
        super().mousePressEvent(fakeEvent)



    def middleMouseButtonRelease(self, event:QMouseEvent):
        """When Middle mouse button was released"""
        fakeEvent = QMouseEvent(event.type(), event.localPos(), event.screenPos(),
                                Qt.LeftButton, event.buttons() & ~Qt.LeftButton, event.modifiers())
        super().mouseReleaseEvent(fakeEvent)
        self.setDragMode(QGraphicsView.RubberBandDrag)


    def leftMouseButtonPress(self, event:QMouseEvent):
        """When Left  mouse button was pressed"""

        # get item which we clicked on
        item = self.getItemAtClick(event)

        # we store the position of last LMB click
        self.last_lmb_click_scene_pos = self.mapToScene(event.pos())

        # if DEBUG: print("LMB Click on", item, self.debug_modifiers(event))

        # logic
        if hasattr(item, "node") or isinstance(item, QDMGraphicsEdge) or item is None:
            if event.modifiers() & Qt.ShiftModifier:
                event.ignore()
                fakeEvent = QMouseEvent(QEvent.MouseButtonPress, event.localPos(), event.screenPos(),
                                        Qt.LeftButton, event.buttons() | Qt.LeftButton,
                                        event.modifiers() | Qt.ControlModifier)
                super().mousePressEvent(fakeEvent)
                return


        if isinstance(item, QDMGraphicsSocket):
            if self.mode == MODE_NOOP:
                self.mode = MODE_EDGE_DRAG
                self.dragging.edgeDragStart(item)
                return

        if self.mode == MODE_EDGE_DRAG:
            res = self.dragging.edgeDragEnd(item)
            if res: return

        if item is None:
            if event.modifiers() & Qt.ControlModifier:
                self.mode = MODE_EDGE_CUT
                fakeEvent = QMouseEvent(QEvent.MouseButtonRelease, event.localPos(), event.screenPos(),
                                        Qt.LeftButton, Qt.NoButton, event.modifiers())
                super().mouseReleaseEvent(fakeEvent)
                QApplication.setOverrideCursor(Qt.CrossCursor)
                return
            else:
                self.rubberBandDraggingRectangle = True

        super().mousePressEvent(event)


    def leftMouseButtonRelease(self, event:QMouseEvent):
        """When Left  mouse button was released"""

        # get item which we release mouse button on
        item = self.getItemAtClick(event)

        try:
            # logic
            if hasattr(item, "node") or isinstance(item, QDMGraphicsEdge) or item is None:
                if event.modifiers() & Qt.ShiftModifier:
                    event.ignore()
                    fakeEvent = QMouseEvent(event.type(), event.localPos(), event.screenPos(),
                                            Qt.LeftButton, Qt.NoButton,
                                            event.modifiers() | Qt.ControlModifier)
                    super().mouseReleaseEvent(fakeEvent)
                    return

            if self.mode == MODE_EDGE_DRAG:
                if self.distanceBetweenClickAndReleaseIsOff(event):
                    res = self.dragging.edgeDragEnd(item)
                    if res: return

            if self.mode == MODE_EDGE_CUT:
                self.cutIntersectingEdges()
                self.cutline.line_points = []
                self.cutline.update()
                QApplication.setOverrideCursor(Qt.ArrowCursor)
                self.mode = MODE_NOOP
                return


            if self.rubberBandDraggingRectangle:
                self.rubberBandDraggingRectangle = False
                current_selected_items = self.grScene.selectedItems()

                if current_selected_items != self.grScene.scene._last_selected_items:
                    if current_selected_items == []:
                        self.grScene.itemsDeselected.emit()
                    else:
                        self.grScene.itemSelected.emit()
                    self.grScene.scene._last_selected_items = current_selected_items

                return

            # otherwise deselect everything
            if item is None:
                self.grScene.itemsDeselected.emit()

        except: dumpException()

        super().mouseReleaseEvent(event)



    def rightMouseButtonPress(self, event:QMouseEvent):
        """When Right mouse button was pressed"""
        super().mousePressEvent(event)


    def rightMouseButtonRelease(self, event:QMouseEvent):
        """When Right mouse button was release"""

        ## cannot be because with dragging RMB we spawn Create New Node Context Menu
        ## However, you could use this if you want to cancel with RMB
        # if self.mode == MODE_EDGE_DRAG:
        #     self.dragging.edgeDragEnd(None)
        #     return

        super().mouseReleaseEvent(event)


    def mouseMoveEvent(self, event:QMouseEvent):
        """Overriden Qt's ``mouseMoveEvent`` handling Scene/View logic"""
        scenepos = self.mapToScene(event.pos())

        if self.mode == MODE_EDGE_DRAG:
            self.dragging.updateDestination(scenepos.x(), scenepos.y())

        if self.mode == MODE_EDGE_CUT and self.cutline is not None:
            self.cutline.line_points.append(scenepos)
            self.cutline.update()

        self.last_scene_mouse_position = scenepos

        self.scenePosChanged.emit( int(scenepos.x()), int(scenepos.y()) )

        super().mouseMoveEvent(event)


    def keyPressEvent(self, event:QKeyEvent):
        """
        .. note::
            This overriden Qt's method was used for handling key shortcuts, before we implemented propper
            ``QWindow`` with Actions and Menu. Still the commented code serves as an example how to handle
            key presses without Qt's framework for Actions and shortcuts. There can be also found an example
            how to solve the problem when Node does contain Text/LineEdit and we press `Delete`
            key (also serving to delete `Node`)

        :param event: Qt's Key event
        :type event: ``QKeyEvent``
        :return:
        """
        # Use this code below if you wanna have shortcuts in this widget.
        # You want to use this, when you don't have a window which handles these shortcuts for you

        # if event.key() == Qt.Key_Delete:
        #     if not self.editingFlag:
        #         self.deleteSelected()
        #     else:
        #         super().keyPressEvent(event)
        # elif event.key() == Qt.Key_S and event.modifiers() & Qt.ControlModifier:
        #     self.grScene.scene.saveToFile("graph.json")
        # elif event.key() == Qt.Key_L and event.modifiers() & Qt.ControlModifier:
        #     self.grScene.scene.loadFromFile("graph.json")
        # elif event.key() == Qt.Key_Z and event.modifiers() & Qt.ControlModifier and not event.modifiers() & Qt.ShiftModifier:
        #     self.grScene.scene.history.undo()
        # elif event.key() == Qt.Key_Z and event.modifiers() & Qt.ControlModifier and event.modifiers() & Qt.ShiftModifier:
        #     self.grScene.scene.history.redo()
        # elif event.key() == Qt.Key_H:
        #     print("HISTORY:     len(%d)" % len(self.grScene.scene.history.history_stack),
        #           " -- current_step", self.grScene.scene.history.history_current_step)
        #     ix = 0
        #     for item in self.grScene.scene.history.history_stack:
        #         print("#", ix, "--", item['desc'])
        #         ix += 1
        # else:
        super().keyPressEvent(event)


    def cutIntersectingEdges(self):
        """Compare which `Edges` intersect with current `Cut line` and delete them safely"""
        for ix in range(len(self.cutline.line_points) - 1):
            p1 = self.cutline.line_points[ix]
            p2 = self.cutline.line_points[ix + 1]

            # @TODO: we could collect all touched nodes, and notify them once after all edges removed
            # we could cut 3 edges leading to a single nodeeditor this will notify it 3x
            # maybe we could use some Notifier class with methods collect() and dispatch()
            for edge in self.grScene.scene.edges.copy():
                if edge.grEdge.intersectsWith(p1, p2):
                    edge.remove()
        self.grScene.scene.history.storeHistory("Delete cutted edges", setModified=True)



    def deleteSelected(self):
        """Shortcut for safe deleting every object selected in the `Scene`."""
        for item in self.grScene.selectedItems():
            if isinstance(item, QDMGraphicsEdge):
                item.edge.remove()
            elif hasattr(item, 'node'):
                item.node.remove()
        self.grScene.scene.history.storeHistory("Delete selected", setModified=True)



    def debug_modifiers(self, event):
        """Helper function get string if we hold Ctrl, Shift or Alt modifier keys"""
        out = "MODS: "
        if event.modifiers() & Qt.ShiftModifier: out += "SHIFT "
        if event.modifiers() & Qt.ControlModifier: out += "CTRL "
        if event.modifiers() & Qt.AltModifier: out += "ALT "
        return out

    def getItemAtClick(self, event:QEvent) -> 'QGraphicsItem':
        """Return the object on which we've clicked/release mouse button

        :param event: Qt's mouse or key event
        :type event: ``QEvent``
        :return: ``QGraphicsItem`` which the mouse event happened or ``None``
        """
        pos = event.pos()
        obj = self.itemAt(pos)
        return obj


    def distanceBetweenClickAndReleaseIsOff(self, event:QMouseEvent) -> bool:
        """ Measures if we are too far from the last Mouse button click scene position.
        This is used for detection if we release too far after we clicked on a `Socket`

        :param event: Qt's mouse event
        :type event: ``QMouseEvent``
        :return: ``True`` if we released too far from where we clicked before
        """
        new_lmb_release_scene_pos = self.mapToScene(event.pos())
        dist_scene = new_lmb_release_scene_pos - self.last_lmb_click_scene_pos
        edge_drag_threshold_sq = EDGE_DRAG_START_TRHESHOLD*EDGE_DRAG_START_TRHESHOLD
        return (dist_scene.x()*dist_scene.x() + dist_scene.y()*dist_scene.y()) > edge_drag_threshold_sq



    def wheelEvent(self, event:QWheelEvent):
        """overriden Qt's ``wheelEvent``. This handles zooming"""
        # calculate our zoom Factor
        zoomOutFactor = 1 / self.zoomInFactor

        # calculate zoom
        if event.angleDelta().y() > 0:
            zoomFactor = self.zoomInFactor
            self.zoom += self.zoomStep
        else:
            zoomFactor = zoomOutFactor
            self.zoom -= self.zoomStep


        clamped = False
        if self.zoom < self.zoomRange[0]: self.zoom, clamped = self.zoomRange[0], True
        if self.zoom > self.zoomRange[1]: self.zoom, clamped = self.zoomRange[1], True

        # set scene scale
        if not clamped or self.zoomClamp is False:
            self.scale(zoomFactor, zoomFactor)




# -*- coding: utf-8 -*-
"""
Created on Sat Sep  5 19:39:26 2020

@author: zohei
"""

from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################
#from Modules_GroupeFrigorifique.Evaporator import Evaporator
#from Modules_GroupeFrigorifique.Compressor import Compressor
from ThermodynamicCycles.Desuperheater import Desuperheater
#from Modules_GroupeFrigorifique.Expansion_Valve import Expansion_Valve
#from Modules_GroupeFrigorifique.Condenser import Condenser
from ThermodynamicCycles.Connect import Fluid_connect
from ThermodynamicCycles.FluidPort.FluidPort import FluidPort




class CalcDesuContent(QDMNodeContentWidget):
    def initUI(self):
        
        # self.subcooling_lbl=QLabel("subcooling", self) 
        # self.subcooling_edit=QLineEdit("0.01", self)
        
        # self.IsenEff_lbl=QLabel("Rendement isentropique", self) 
        # self.IsenEff_edit=QLineEdit("0.7", self)
        
        
        
        # self.Tref_lbl=QLabel("refoidissement cible (°C)", self) 
        # self.Tref_edit=QLineEdit("80", self)
        
        self.Tdesu_lbl_title = QLabel("Tcond(°C)", self)
        self.Tdesu_lbl = QLabel("", self)
        
        self.Qdesu_lbl_title = QLabel("Qdesu (kW):", self)
        self.Qdesu_lbl = QLabel("", self)
        
        # self.Tis_lbl_title = QLabel("Temp. isentrop. (°C)", self)
        # self.Tis_lbl = QLabel("", self)
        
        
        
        
         
        self.layout=QVBoxLayout()

        # self.layout.addWidget(self.subcooling_lbl)
        # self.layout.addWidget(self.subcooling_edit) 
        
        # self.layout.addWidget(self.Tref_lbl)
        # self.layout.addWidget(self.Tref_edit)
        
        # self.layout.addWidget(self.IsenEff_lbl)
        # self.layout.addWidget(self.IsenEff_edit)
        
        
        
        self.layout.addWidget(self.Tdesu_lbl_title)
        self.layout.addWidget(self.Tdesu_lbl)  
        
        self.layout.addWidget(self.Qdesu_lbl_title)
        self.layout.addWidget(self.Qdesu_lbl) 
      
        # self.layout.addWidget(self.Tis_lbl_title)
        # self.layout.addWidget(self.Tis_lbl)
        
     
        
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        
        
    def serialize(self):
        # res = super().serialize()
        # res['value'] = self.subcooling_edit.text()
    
        
        # return res #,res2,res3,res4
        
        pass

    def deserialize(self, data, hashmap={}):
        # res = super().deserialize(data, hashmap)
        
        # try:
            
        #     value = data["value"]
        
            
        #     self.subcooling_edit.setText(value)
           
            
        #     return True & res # & res2 & res3 & res4
        # except Exception as e:
        #     dumpException(e)
        # return res #,res2,res3,res4
        pass

@register_node(OP_NODE_DESU)
class CalcNode_Desu(CalcNode):
    icon = "icons/desuperheater.png"
    op_code = OP_NODE_DESU
    op_title = "Désurchauffeur"
    content_label = "/"
    content_label_objname = "calc_node_desu"
    
    def __init__(self, scene):
        super().__init__(scene, inputs=[2], outputs=[1])
        self.eval()
        
    def initInnerClasses(self):
        self.content = CalcDesuContent(self)
        self.grNode = CalcGraphicsNode(self)
        
        
        self.grNode.height=150
        self.grNode.width=200
        
        # self.content.subcooling_edit.textChanged.connect(self.onInputChanged)
        
  
    def evalOperation(self, input1, input2):
        self.value=[]
        
        a=FluidPort()
        a.fluid=input1[0]
        a.F=input1[1]
        a.P=input1[2]*1e5
        a.h=input1[3]*1000
       
             
                #####Désurchauffeur    
             
        DESU=Desuperheater.Object()
        Fluid_connect(DESU.Inlet,a)
        DESU.calculate()
       
        self.value.append(DESU.Outlet.fluid) #fluide
        self.value.append(DESU.Outlet.F) #débit
        self.value.append(DESU.Outlet.P/1e5) #pression min
        self.value.append(DESU.Outlet.h/1000) #Enthalpie
        
        # self.content.fluid_lbl.setText("%f" % (COMP.Q_comp/1000)) #"%d" % 
        self.content.Qdesu_lbl.setText("%f" % (DESU.Qdesurch/1000))
        self.content.Tdesu_lbl.setText("%f" % (DESU.Tsv-273.15))
        
        #self.content.lbl.setText("%f" % val[3])
        
        
            
        return self.value


# -*- coding: utf-8 -*-
"""
A module containing Main Window class
"""
import os
import json
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from NodeEditor.nodeeditor.node_editor_widget import NodeEditorWidget


class NodeEditorWindow(QMainWindow):
    NodeEditorWidget_class = NodeEditorWidget

    """Class representing NodeEditor's Main Window"""
    def __init__(self):
        """
        :Instance Attributes:

        - **name_company** - name of the company, used for permanent profile settings
        - **name_product** - name of this App, used for permanent profile settings
        """
        super().__init__()

        self.name_company = 'Blenderfreak'
        self.name_product = 'NodeEditor'

        self.initUI()


    def initUI(self):
        """Set up this ``QMainWindow``. Create :class:`~nodeeditor.node_editor_widget.NodeEditorWidget`, Actions and Menus"""
        self.createActions()
        self.createMenus()

        # create node editor widget
        self.nodeeditor = self.__class__.NodeEditorWidget_class(self)
        self.nodeeditor.scene.addHasBeenModifiedListener(self.setTitle)
        self.setCentralWidget(self.nodeeditor)

        self.createStatusBar()

        # set window properties
        # self.setGeometry(200, 200, 800, 600)
        self.setTitle()
        self.show()

    def sizeHint(self):
        return QSize(800, 600)

    def createStatusBar(self):
        """Create Status bar and connect to `Graphics View` scenePosChanged event"""
        self.statusBar().showMessage("")
        self.status_mouse_pos = QLabel("")
        self.statusBar().addPermanentWidget(self.status_mouse_pos)
        self.nodeeditor.view.scenePosChanged.connect(self.onScenePosChanged)

    def createActions(self):
        """Create basic `File` and `Edit` actions"""
        self.actNew = QAction('&New', self, shortcut='Ctrl+N', statusTip="Create new graph", triggered=self.onFileNew)
        self.actOpen = QAction('&Open', self, shortcut='Ctrl+O', statusTip="Open file", triggered=self.onFileOpen)
        self.actSave = QAction('&Save', self, shortcut='Ctrl+S', statusTip="Save file", triggered=self.onFileSave)
        self.actSaveAs = QAction('Save &As...', self, shortcut='Ctrl+Shift+S', statusTip="Save file as...", triggered=self.onFileSaveAs)
        self.actExit = QAction('E&xit', self, shortcut='Ctrl+Q', statusTip="Exit application", triggered=self.close)

        self.actUndo = QAction('&Undo', self, shortcut='Ctrl+Z', statusTip="Undo last operation", triggered=self.onEditUndo)
        self.actRedo = QAction('&Redo', self, shortcut='Ctrl+Shift+Z', statusTip="Redo last operation", triggered=self.onEditRedo)
        self.actCut = QAction('Cu&t', self, shortcut='Ctrl+X', statusTip="Cut to clipboard", triggered=self.onEditCut)
        self.actCopy = QAction('&Copy', self, shortcut='Ctrl+C', statusTip="Copy to clipboard", triggered=self.onEditCopy)
        self.actPaste = QAction('&Paste', self, shortcut='Ctrl+V', statusTip="Paste from clipboard", triggered=self.onEditPaste)
        self.actDelete = QAction('&Delete', self, shortcut='Del', statusTip="Delete selected items", triggered=self.onEditDelete)


    def createMenus(self):
        """Create Menus for `File` and `Edit`"""
        self.createFileMenu()
        self.createEditMenu()

    def createFileMenu(self):
        menubar = self.menuBar()
        self.fileMenu = menubar.addMenu('&File')
        self.fileMenu.addAction(self.actNew)
        self.fileMenu.addSeparator()
        self.fileMenu.addAction(self.actOpen)
        self.fileMenu.addAction(self.actSave)
        self.fileMenu.addAction(self.actSaveAs)
        self.fileMenu.addSeparator()
        self.fileMenu.addAction(self.actExit)

    def createEditMenu(self):
        menubar = self.menuBar()
        self.editMenu = menubar.addMenu('&Edit')
        self.editMenu.addAction(self.actUndo)
        self.editMenu.addAction(self.actRedo)
        self.editMenu.addSeparator()
        self.editMenu.addAction(self.actCut)
        self.editMenu.addAction(self.actCopy)
        self.editMenu.addAction(self.actPaste)
        self.editMenu.addSeparator()
        self.editMenu.addAction(self.actDelete)

    def setTitle(self):
        """Function responsible for setting window title"""
        title = "Node Editor - "
        title += self.getCurrentNodeEditorWidget().getUserFriendlyFilename()

        self.setWindowTitle(title)


    def closeEvent(self, event):
        """Handle close event. Ask before we loose work"""
        if self.maybeSave():
            event.accept()
        else:
            event.ignore()

    def isModified(self) -> bool:
        """Has current :class:`~nodeeditor.node_scene.Scene` been modified?

        :return: ``True`` if current :class:`~nodeeditor.node_scene.Scene` has been modified
        :rtype: ``bool``
        """
        nodeeditor = self.getCurrentNodeEditorWidget()
        return nodeeditor.scene.isModified() if nodeeditor else False

    def getCurrentNodeEditorWidget(self) -> NodeEditorWidget:
        """get current :class:`~nodeeditor.node_editor_widget`

        :return: get current :class:`~nodeeditor.node_editor_widget`
        :rtype: :class:`~nodeeditor.node_editor_widget`
        """
        return self.centralWidget()

    def maybeSave(self) -> bool:
        """If current `Scene` is modified, ask a dialog to save the changes. Used before
        closing window / mdi child document

        :return: ``True`` if we can continue in the `Close Event` and shutdown. ``False`` if we should cancel
        :rtype: ``bool``
        """
        if not self.isModified():
            return True

        res = QMessageBox.warning(self, "About to loose your work?",
                "The document has been modified.\n Do you want to save your changes?",
                QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel
              )

        if res == QMessageBox.Save:
            return self.onFileSave()
        elif res == QMessageBox.Cancel:
            return False

        return True


    def onScenePosChanged(self, x:int, y:int):
        """Handle event when cursor position changed on the `Scene`

        :param x: new cursor x position
        :type x:
        :param y: new cursor y position
        :type y:
        """
        self.status_mouse_pos.setText("Scene Pos: [%d, %d]" % (x, y))

    def getFileDialogDirectory(self):
        """Returns starting directory for ``QFileDialog`` file open/save"""
        return ''

    def getFileDialogFilter(self):
        """Returns ``str`` standard file open/save filter for ``QFileDialog``"""
        return 'Graph (*.json);;All files (*)'

    def onFileNew(self):
        """Hande File New operation"""
        if self.maybeSave():
            self.getCurrentNodeEditorWidget().fileNew()
            self.setTitle()


    def onFileOpen(self):
        """Handle File Open operation"""
        if self.maybeSave():
            fname, filter = QFileDialog.getOpenFileName(self, 'Open graph from file', self.getFileDialogDirectory(), self.getFileDialogFilter())
            if fname != '' and os.path.isfile(fname):
                self.getCurrentNodeEditorWidget().fileLoad(fname)
                self.setTitle()

    def onFileSave(self):
        """Handle File Save operation"""
        current_nodeeditor = self.getCurrentNodeEditorWidget()
        if current_nodeeditor is not None:
            if not current_nodeeditor.isFilenameSet(): return self.onFileSaveAs()

            current_nodeeditor.fileSave()
            self.statusBar().showMessage("Successfully saved %s" % current_nodeeditor.filename, 5000)

            # support for MDI app
            if hasattr(current_nodeeditor, "setTitle"): current_nodeeditor.setTitle()
            else: self.setTitle()
            return True

    def onFileSaveAs(self):
        """Handle File Save As operation"""
        current_nodeeditor = self.getCurrentNodeEditorWidget()
        if current_nodeeditor is not None:
            fname, filter = QFileDialog.getSaveFileName(self, 'Save graph to file', self.getFileDialogDirectory(), self.getFileDialogFilter())
            if fname == '': return False

            self.onBeforeSaveAs(current_nodeeditor, fname)
            current_nodeeditor.fileSave(fname)
            self.statusBar().showMessage("Successfully saved as %s" % current_nodeeditor.filename, 5000)

            # support for MDI app
            if hasattr(current_nodeeditor, "setTitle"): current_nodeeditor.setTitle()
            else: self.setTitle()
            return True

    def onBeforeSaveAs(self, current_nodeeditor: 'NodeEditorWidget', filename: str):
        """
        Event triggered after choosing filename and before actual fileSave(). We are passing current_nodeeditor because
        we will loose focus after asking with QFileDialog and therefore getCurrentNodeEditorWidget will return None
        """
        pass

    def onEditUndo(self):
        """Handle Edit Undo operation"""
        if self.getCurrentNodeEditorWidget():
            self.getCurrentNodeEditorWidget().scene.history.undo()

    def onEditRedo(self):
        """Handle Edit Redo operation"""
        if self.getCurrentNodeEditorWidget():
            self.getCurrentNodeEditorWidget().scene.history.redo()

    def onEditDelete(self):
        """Handle Delete Selected operation"""
        if self.getCurrentNodeEditorWidget():
            self.getCurrentNodeEditorWidget().scene.getView().deleteSelected()

    def onEditCut(self):
        """Handle Edit Cut to clipboard operation"""
        if self.getCurrentNodeEditorWidget():
            data = self.getCurrentNodeEditorWidget().scene.clipboard.serializeSelected(delete=True)
            str_data = json.dumps(data, indent=4)
            QApplication.instance().clipboard().setText(str_data)

    def onEditCopy(self):
        """Handle Edit Copy to clipboard operation"""
        if self.getCurrentNodeEditorWidget():
            data = self.getCurrentNodeEditorWidget().scene.clipboard.serializeSelected(delete=False)
            str_data = json.dumps(data, indent=4)
            QApplication.instance().clipboard().setText(str_data)

    def onEditPaste(self):
        """Handle Edit Paste from clipboard operation"""
        if self.getCurrentNodeEditorWidget():
            raw_data = QApplication.instance().clipboard().text()

            try:
                data = json.loads(raw_data)
            except ValueError as e:
                print("Pasting of not valid json data!", e)
                return

            # check if the json data are correct
            if 'nodes' not in data:
                print("JSON does not contain any nodes!")
                return

            return self.getCurrentNodeEditorWidget().scene.clipboard.deserializeFromClipboard(data)

    def readSettings(self):
        """Read the permanent profile settings for this app"""
        settings = QSettings(self.name_company, self.name_product)
        pos = settings.value('pos', QPoint(200, 200))
        size = settings.value('size', QSize(400, 400))
        self.move(pos)
        self.resize(size)

    def writeSettings(self):
        """Write the permanent profile settings for this app"""
        settings = QSettings(self.name_company, self.name_product)
        settings.setValue('pos', self.pos())
        settings.setValue('size', self.size())


"""
Script de diagnostic détaillé pour tester la clé API OpenWeatherMap
"""

import sys
import os
import requests

# Ajouter le chemin du module OpenWeatherMap
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'src', 'OpenWeatherMap'))

import get_weather

def test_api_detailed():
    print("=" * 80)
    print("DIAGNOSTIC DÉTAILLÉ DE LA CLÉ API OPENWEATHERMAP")
    print("=" * 80)
    
    # 1. Récupérer la clé API
    print("\n📋 ÉTAPE 1 : Vérification de la configuration")
    print("-" * 80)
    try:
        api_key = get_weather.get_api_key()
        print(f"✓ Clé API récupérée depuis config.ini")
        print(f"  Clé complète : {api_key}")
        print(f"  Longueur     : {len(api_key)} caractères")
        
        # Vérifier le format
        if len(api_key) != 32:
            print(f"⚠️  ATTENTION : La clé devrait avoir 32 caractères, elle en a {len(api_key)}")
        if not api_key.isalnum():
            print(f"⚠️  ATTENTION : La clé contient des caractères non alphanumériques")
    except Exception as e:
        print(f"✗ Erreur : {e}")
        return False
    
    # 2. Test direct de l'API
    print("\n🌐 ÉTAPE 2 : Test direct de l'API OpenWeatherMap")
    print("-" * 80)
    
    # Test avec Paris
    lat, lon = 48.8566, 2.3522
    url = f"http://api.openweathermap.org/data/2.5/weather?lat={lat}&lon={lon}&appid={api_key}"
    
    print(f"URL testée : {url}")
    print("\n⏳ Envoi de la requête...")
    
    try:
        response = requests.get(url, timeout=10)
        print(f"\n📊 Réponse reçue :")
        print(f"  Code HTTP : {response.status_code}")
        print(f"  Statut    : {response.reason}")
        
        if response.status_code == 200:
            print("\n✅ SUCCÈS ! La clé API fonctionne correctement.")
            data = response.json()
            print("\n📍 Données météo reçues pour Paris :")
            print(f"  Ville       : {data.get('name', 'N/A')}")
            print(f"  Pays        : {data.get('sys', {}).get('country', 'N/A')}")
            print(f"  Température : {data.get('main', {}).get('temp', 0) - 273.15:.1f} °C")
            print(f"  Ressenti    : {data.get('main', {}).get('feels_like', 0) - 273.15:.1f} °C")
            print(f"  Description : {data.get('weather', [{}])[0].get('description', 'N/A')}")
            print(f"  Humidité    : {data.get('main', {}).get('humidity', 'N/A')} %")
            print(f"  Pression    : {data.get('main', {}).get('pressure', 'N/A')} hPa")
            print(f"  Vent        : {data.get('wind', {}).get('speed', 'N/A')} m/s")
            print(f"  Nuages      : {data.get('clouds', {}).get('all', 'N/A')} %")
            return True
            
        elif response.status_code == 401:
            print("\n❌ ERREUR 401 : CLÉ API NON AUTORISÉE")
            try:
                error_data = response.json()
                print(f"\n📝 Message d'erreur de l'API :")
                print(f"  {error_data.get('message', 'Pas de message')}")
            except:
                print(f"  Contenu brut : {response.text}")
            
            print("\n🔍 Causes possibles :")
            print("  1. La clé API n'est pas encore activée (peut prendre quelques heures)")
            print("  2. La clé API est invalide ou a expiré")
            print("  3. La clé API n'a pas les permissions nécessaires")
            print("  4. Le compte OpenWeatherMap a été suspendu")
            
            print("\n💡 Solutions recommandées :")
            print("  1. Vérifiez votre compte sur https://home.openweathermap.org/api_keys")
            print("  2. Assurez-vous que la clé est 'Active'")
            print("  3. Si la clé vient d'être créée, attendez 1-2 heures")
            print("  4. Générez une nouvelle clé API si nécessaire")
            return False
            
        elif response.status_code == 404:
            print("\n❌ ERREUR 404 : Localisation non trouvée")
            return False
            
        elif response.status_code == 429:
            print("\n❌ ERREUR 429 : Limite de requêtes dépassée")
            print("  Trop de requêtes ont été effectuées. Attendez quelques minutes.")
            return False
            
        else:
            print(f"\n❌ ERREUR {response.status_code} : {response.reason}")
            try:
                print(f"  Message : {response.json()}")
            except:
                print(f"  Contenu : {response.text}")
            return False
            
    except requests.exceptions.Timeout:
        print("\n❌ ERREUR : Timeout de connexion")
        print("  La requête a pris trop de temps. Vérifiez votre connexion Internet.")
        return False
        
    except requests.exceptions.ConnectionError:
        print("\n❌ ERREUR : Impossible de se connecter à l'API")
        print("  Vérifiez votre connexion Internet.")
        return False
        
    except Exception as e:
        print(f"\n❌ ERREUR INATTENDUE : {e}")
        import traceback
        traceback.print_exc()
        return False
    
    # 3. Test avec l'ancienne clé pour comparaison
    print("\n🔄 ÉTAPE 3 : Test avec l'ancienne clé (pour comparaison)")
    print("-" * 80)
    old_api_key = "7b18d48745ae006628ff1ee161be2b69"
    url_old = f"http://api.openweathermap.org/data/2.5/weather?lat={lat}&lon={lon}&appid={old_api_key}"
    
    print(f"Test de l'ancienne clé : {old_api_key[:10]}...{old_api_key[-10:]}")
    
    try:
        response_old = requests.get(url_old, timeout=10)
        print(f"  Code HTTP de l'ancienne clé : {response_old.status_code}")
        
        if response_old.status_code == 200:
            print("  ✓ L'ancienne clé fonctionne toujours")
        elif response_old.status_code == 401:
            print("  ✗ L'ancienne clé ne fonctionne plus non plus")
        else:
            print(f"  ? L'ancienne clé retourne : {response_old.status_code}")
    except Exception as e:
        print(f"  Erreur lors du test de l'ancienne clé : {e}")

if __name__ == "__main__":
    print("\n")
    try:
        success = test_api_detailed()
        print("\n" + "=" * 80)
        if success:
            print("✅ DIAGNOSTIC TERMINÉ : La clé API fonctionne correctement !")
        else:
            print("❌ DIAGNOSTIC TERMINÉ : La clé API ne fonctionne pas correctement")
        print("=" * 80)
        print("\n")
    except KeyboardInterrupt:
        print("\n\n⚠️  Diagnostic interrompu par l'utilisateur")
        sys.exit(1)



import CoolProp.CoolProp as CP
import matplotlib.pyplot as plt
import numpy as np

class Object:
    def __init__(self, fluid):
        self.fluid = fluid
        self.T_min = CP.PropsSI(fluid, 'Ttriple') - 273.15  # Triple point temperature in Celsius
        self.T_crit = CP.PropsSI(fluid, 'Tcrit') - 273.15   # Critical temperature in Celsius
        self.T_max = 2.5 * self.T_crit                      # 2.5 times critical temperature
        self.S_min = None
        self.S_max = None
        self.custom_points = []  # List to store custom points

    def set_temperature_range(self, T_min=None, T_max=None):
        if T_min is not None:
            self.T_min = T_min
        if T_max is not None:
            self.T_max = T_max

    def set_entropy_range(self, S_min=None, S_max=None):
        self.S_min = S_min
        self.S_max = S_max

    def add_points(self, points):
        for point in points:
            self.custom_points.append({'T': point['T'], 'S': point['S']})  # Temperature in Celsius

    def show(self, draw_arrows=False,figsize=(10, 6)):
        T_range = np.linspace(self.T_min, self.T_max, int(self.T_max-self.T_min)*5)
        plt.figure(figsize=figsize)

        # Plot saturation curves
        S_liquid = [CP.PropsSI('S', 'T', T + 273.15, 'Q', 0, self.fluid) for T in T_range if T <= self.T_crit]
        S_vapor = [CP.PropsSI('S', 'T', T + 273.15, 'Q', 1, self.fluid) for T in T_range if T <= self.T_crit]
        plt.plot(S_liquid, T_range[:len(S_liquid)], 'b')  # T in °C
        plt.plot(S_vapor, T_range[:len(S_vapor)], 'r')  # T in °C

        # Adjust S range if specified
        if self.S_min is not None or self.S_max is not None:
            plt.xlim(self.S_min, self.S_max)

        # Define pressures for isobar lines
        p_crit = CP.PropsSI(self.fluid, 'pcrit')
        pressures = [0.1 * 100000]+[0.5 * 100000] + list(np.arange(1 * 100000, 10 * 100000, 1 * 100000)) + \
                    list(np.arange(10 * 100000, 2 * p_crit + 1, 10 * 100000))

# Plot lines of constant pressure with spaced labels
# Plot lines of constant pressure with spaced labels
        n_pressures = len(pressures)
        for i, p in enumerate(pressures):
            S = []
            for T in T_range:
                try:
                    T_K = T + 273.15  # Convert to Kelvin
                    entropy = CP.PropsSI('S', 'P', p, 'T', T_K, self.fluid)
                    S.append(entropy)
                except ValueError:
                    continue
            if S:
                is_ten_bar_multiple = int(p / 100000) % 10 == 0 or p == 0.1 * 100000
                line_color = 'black' if is_ten_bar_multiple else 'blue'
                plt.plot(S, T_range[:len(S)], '--', linewidth=0.5, color=line_color)
                if is_ten_bar_multiple:
                    # Place label at a unique vertical position for each pressure line
                    label_pos = len(S) * i // n_pressures
                    plt.annotate(f'{p/100000:.1f} bar',  # Convert to bar
                                xy=(S[label_pos], T_range[label_pos]), 
                                xytext=(5, -15), textcoords='offset points', 
                                color=line_color, ha='right', fontsize=6)



        # Plot custom points and optionally arrows
        for i in range(len(self.custom_points) - 1):
            point = self.custom_points[i]
            next_point = self.custom_points[i + 1]
            plt.scatter(point['S'], point['T'], color='red')
            if draw_arrows:
                plt.arrow(point['S'], point['T'], next_point['S'] - point['S'], next_point['T'] - point['T'],
                          head_width=0.05, head_length=0.1, fc='black', ec='black', linewidth=0.5)

        if self.custom_points:
            last_point = self.custom_points[-1]
            plt.scatter(last_point['S'], last_point['T'], color='red')

        # Label the axes, title, and grid
        plt.xlabel('Entropy (J/kg-K)')
        plt.ylabel('Temperature (°C)')
        plt.title('Temperature-Entropy Diagram for ' + self.fluid)
        plt.grid(True)

        # Show the plot
        plt.show()

# Example usage
# Chart = Object('R407C')
# Chart.add_points([{'S': 1000, 'T': 12}, {'S': 666, 'T': 12}, {'S': 2000, 'T': 15}])
# Chart.show(draw_arrows=True)



# Importation des modules nécessaires
from AHU.air_humide import air_humide
from AHU.air_humide import air_humide_NB
from AHU.AirPort.AirPort import AirPort
import pandas as pd

class Object:
    """
    Classe représentant une unité de traitement d'air avec des propriétés thermodynamiques
    et des calculs associés à l'air humide. Elle calcule les propriétés de l'air en fonction
    de la température, de l'humidité relative, du débit d'air et de la pression.
    """
    
    def __init__(self):
        """
        Initialisation de l'objet. Définit les propriétés de l'air et initialise les objets
        AirPort pour l'entrée et la sortie de l'air.
        """
        self.Inlet = AirPort()  # Port d'entrée de l'air
        self.Outlet = AirPort()  # Port de sortie de l'air
        self.id = 1  # Identifiant de l'objet
        self.T = None  # Température de l'air (Celsius)
        self.RH = None  # Humidité relative de l'air (%)
        self.F = None  # Débit massique d'air (kg/s)
        self.Pv_sat = 0  # Pression de vapeur saturante (Pa)
        self.w = 0  # Humidité absolue de l'air (kg/kg)
        self.T_hum = 0  # Température humide (C)
        self.h = 0  # Enthalpie spécifique (J/kg)
        self.P = 101325  # Pression de l'air (Pa) - valeur standard à la mer
        self.F_m3h = None  # Débit volumique en m3/h (air humide)
        self.F_dry = None  # Débit d'air sec (kg/s)
        self.df = None  # DataFrame pour afficher les résultats

    def calculate(self):
        """
        Effectue les calculs thermodynamiques sur l'air humide. Calcule l'humidité absolue,
        l'enthalpie, le débit d'air sec et met à jour les propriétés d'entrée et de sortie.
        """
        
        # Si la pression d'entrée est spécifiée, on l'utilise pour les calculs
        if self.Inlet.P is not None:
            self.P = self.Inlet.P
        
        # Cas où T et RH sont définis
        if self.T is not None and self.RH is not None:
            #print(f"Calcul avec T={self.T}°C, RH={self.RH}%")  # Affichage des valeurs d'entrée
            # Calcul de la pression de vapeur saturante à partir de la température
            self.Pv_sat = air_humide.Air_Pv_sat(self.T)
            # Calcul de l'humidité absolue en fonction de la pression de vapeur saturante, de l'humidité relative et de la pression
            self.w = air_humide.Air_w(Pv_sat=self.Pv_sat, RH=self.RH, P=self.P)
            # Calcul de l'enthalpie spécifique de l'air humide à partir de la température et de l'humidité absolue
            self.h = air_humide.Air_h(T_db=self.T, w=self.w)
            # Mise à jour de l'humidité absolue et de l'enthalpie à l'entrée
            self.Inlet.w = self.w
            self.Inlet.h = self.h

            # Si un débit volumique est fourni, on le convertit en débit massique (kg/s)
            if self.F_m3h is not None:
                self.F = self.F_m3h * air_humide.Air_rho_hum(self.T, self.RH, self.P) / 3600  # Conversion m3/h à kg/s
        
        # Cas où seulement le débit d'air est disponible
        if self.Inlet.F is not None:
            self.F = self.Inlet.F
        
        # Si le débit massique est disponible, on l'affecte à l'entrée
        if self.F is not None:
            self.Inlet.F = self.F
        
        # Mise à jour de l'enthalpie et de l'humidité absolue à partir des valeurs de l'entrée
        if self.Inlet.h is not None:
            self.h = self.Inlet.h
        if self.Inlet.w is not None:
            self.w = self.Inlet.w
        
        # Calcul du débit d'air sec en fonction de l'air humide et de l'humidité absolue
        self.F_dry = self.F / (1 + (self.w / 1000))
        if self.Inlet.F_dry is None:
            self.Inlet.F_dry=self.F_dry
        self.Outlet.F_dry=self.F_dry
        # Mise à jour des propriétés de l'air à la sortie en fonction de l'entrée
        self.Outlet.w = self.Inlet.w
        self.Outlet.P = self.Inlet.P
        self.Outlet.h = self.Inlet.h
        self.Outlet.F = self.Inlet.F

        # **Forcer la mise à jour des propriétés des ports (Inlet et Outlet)**
        self.Inlet.update_properties()  # Mise à jour forcée de l'entrée
        self.Outlet.update_properties()  # Mise à jour forcée de la sortie

        # Préparation des données pour créer un DataFrame et afficher les résultats
        self.df = pd.DataFrame({'FreshAir': [
                self.id, round(self.Outlet.T, 1), round(self.Outlet.RH, 1), 
                round(self.Outlet.F, 3), round(self.Outlet.F_dry, 3), 
                round(self.Outlet.P, 1), round(self.Outlet.P / 100000, 1), 
                round(self.Outlet.h, 1), round(self.Outlet.w, 3), 
                round(self.Outlet.Pv_sat, 1)], },
                      index = ['ID', 'Outlet.T (C)', 'Outlet.RH (%)', 'Outlet.F (kg/s)', 
                'Outlet.F_dry (kg/s)', 'Outlet.P (Pa)', 'Outlet.P/10^5 (bar)', 
                'Outlet.h (kJ/kg)', 'Outlet.w (g/kgdry)', 'Outlet.Pv_sat (Pa)'])





from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 

import numpy as np

class Object:
    def __init__(self):
        self.Inlet=FluidPort() 
        self.Ti_degC=None #Tevap
        self.LP_bar=None
        self.fluid = None
        self.Outlet=FluidPort()
        
        self.surchauff=2 #surchauffe
        self.Tl_sat=0
        self.Tsv=0
        self.Hsv=0
        self.Ssv=0
        self.To=0
        self.Ho=0
        self.So=0
        self.Q_evap=0
        
        self.Qevap_i=[] #descritisation de la puissance de l'évaporateur
        self.Tfluid_i=[]
        
        
        #calcul côté eau
        # self.Tw_inlet=30 #°C donnéesd'entrée
        # self.Tw_outlet=20 #°C recalculée
        # self.m_water_flow=1000000000000000000 #m3/h donnée d'entée
        # self.h_water_outlet=0
        # self.h_water_inlet=0
        # self.Twater_i=[]
        
        #calcul de pincement
        #◙self.pinch=[]

        #output data
        self.df=[]
        
    def calculate (self):
        if self.fluid is None:
          self.fluid=self.Inlet.fluid
        else:
          self.Inlet.fluid=self.fluid
       # print("evaporateur P1",self.Inlet.P)

        # Pevap calculation
        if self.Ti_degC is not None:
          self.Inlet.P=PropsSI('P','T',self.Ti_degC+273.15,'Q',0,self.fluid)
        if self.LP_bar is not None:
          self.Inlet.P=1e5*self.LP_bar
        
        self.Tsv = PropsSI('T','P',self.Inlet.P,'Q',1,self.fluid)
        self.Tl_sat=PropsSI('T','P',self.Inlet.P,'Q',0,self.fluid)
        
       # print("evaporateur T1",self.Ti-273.15)
        self.Hsv=PropsSI('H','P',self.Inlet.P,'Q',1,self.fluid)
        self.Ssv=PropsSI('S','P',self.Inlet.P,'Q',1,self.fluid)

        self.To=self.Tsv+self.surchauff
        self.Ho = PropsSI('H','P',self.Inlet.P,'T',self.To,self.fluid)
        self.So = PropsSI('S','P',self.Inlet.P,'T',self.To,self.fluid)
        self.Outlet.fluid=self.fluid 
        self.Outlet.h=self.Ho
        self.Outlet.F=self.Inlet.F
        self.Outlet.P=self.Inlet.P
        
      #  Q_evap avec surchauffe
        self.Q_evap=-self.Inlet.F*(self.Inlet.h-self.Outlet.h)
        
      #  print("self.Inlet.h=",self.Inlet.h)
        
        #print("self.Q_evap=",self.Q_evap)
        if self.Q_evap!=0:
            self.Qevap_i=np.arange(0,self.Q_evap,(self.Q_evap/20)) #descritisation de la puissance de l'évaporateur
        #print("self.Qevapi (kW)=",self.Qevap_i)
      
        
          #bilan sur l'eau
        
        # self.h_water_inlet=PropsSI('H','P',101325,'T',self.Tw_inlet+273.15,"water")
        # print("self.h_water_inlet=",self.h_water_inlet)
        # self.h_water_outlet=(-self.Q_evap/(self.m_water_flow/3.6))+self.h_water_inlet
        # print("self.h_water_outlet=",self.h_water_outlet)
        # self.Tw_outlet=PropsSI('T','P',101325,'H',self.h_water_outlet,"water")-273.15
        # print("self.Tw_outlet=",self.Tw_outlet)
        
        if len(self.Qevap_i)>=20 and self.Q_evap!=0: 
            #calcul descrtisation de la temp du fluide frigorigene
            for i in range(len(self.Qevap_i)):
                self.Tfluid_i.append(0)
            
            for i in range(0,len(self.Qevap_i),1):
                self.Tfluid_i[i]=PropsSI('T','H',self.Inlet.h+self.Qevap_i[i]/self.Inlet.F,'P',self.Inlet.P,self.fluid)
            #print("self.Tfluid_i=",self.Tfluid_i)
            
            #calcul descrtisation de la temp de l'eau   
            
        #     for i in range(len(self.Qevap_i)):
        #         self.Twater_i.append(0)
            
        #     for i in range(0,len(self.Qevap_i),1):
               
        #         self.Twater_i[i]=PropsSI('T','H',self.h_water_outlet+self.Qevap_i[i]/(self.m_water_flow/3.6),'P',101325,"water")
        #     print("self.Twater_i=",self.Twater_i)
            
        #     for i in range(len(self.Qevap_i)):
        #         self.pinch.append(0)
        #         self.pinch[i]=self.Twater_i[i]-self.Tfluid_i[i]
        # print("self.pinch=",self.pinch)

        # Results
        self.Outlet.calculate_properties()
        self.df = pd.DataFrame({'Evaporator': [self.fluid,self.Outlet.F,self.Inlet.P/100000,self.Tsv-273.15,self.Hsv/1000, self.Ssv/1000, self.To-273.15,self.Ho/1000,self.So/1000,self.Q_evap/1000], },
                      index = ['fluid','Outlet.F','Pevap(bar)','Tsv(°C)', 'Hsv(kJ/kg)', 'Ssv(kJ/kg-K)','To(°C)', 'Ho(kJ/kg)', 'So(kJ/kg-K)','Q_evap(kW)',])
       
     

        


# -*- coding: utf-8 -*-
"""
A module containing ``NodeEditorWidget`` class
"""
import os
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *

from NodeEditor.nodeeditor.node_scene import Scene, InvalidFile
from NodeEditor.nodeeditor.node_node import Node
from NodeEditor.nodeeditor.node_edge import Edge, EDGE_TYPE_BEZIER
from NodeEditor.nodeeditor.node_graphics_view import QDMGraphicsView
from NodeEditor.nodeeditor.utils import dumpException


class NodeEditorWidget(QWidget):
    Scene_class = Scene
    GraphicsView_class = QDMGraphicsView

    """The ``NodeEditorWidget`` class"""
    def __init__(self, parent:QWidget=None):
        """
        :param parent: parent widget
        :type parent: ``QWidget``

        :Instance Attributes:

        - **filename** - currently graph's filename or ``None``
        """
        super().__init__(parent)

        self.filename = None

        self.initUI()


    def initUI(self):
        """Set up this ``NodeEditorWidget`` with its layout,  :class:`~nodeeditor.node_scene.Scene` and
        :class:`~nodeeditor.node_graphics_view.QDMGraphicsView`"""
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.layout)

        # crate graphics scene
        self.scene = self.__class__.Scene_class()

        # create graphics view
        self.view = self.__class__.GraphicsView_class(self.scene.grScene, self)
        self.layout.addWidget(self.view)


    def isModified(self) -> bool:
        """Has the `Scene` been modified?

        :return: ``True`` if the `Scene` has been modified
        :rtype: ``bool``
        """
        return self.scene.isModified()

    def isFilenameSet(self) -> bool:
        """Do we have graph loaded from file or new one?

        :return: ``True`` if filename is set. ``False`` if its a graph not saved to a file
        :rtype: ''bool''
        """
        return self.filename is not None

    def getSelectedItems(self) -> list:
        """Shortcut returning `Scene`'s currently selected items

        :return: list of ``QGraphicsItems``
        :rtype: list[QGraphicsItem]
        """
        return self.scene.getSelectedItems()

    def hasSelectedItems(self) -> bool:
        """Is there something selected in the :class:`nodeeditor.node_scene.Scene`?

        :return: ``True`` if there is something selected in the `Scene`
        :rtype: ``bool``
        """
        return self.getSelectedItems() != []

    def canUndo(self) -> bool:
        """Can Undo be performed right now?

        :return: ``True`` if we can undo
        :rtype: ``bool``
        """
        return self.scene.history.canUndo()

    def canRedo(self) -> bool:
        """Can Redo be performed right now?

        :return: ``True`` if we can redo
        :rtype: ``bool``
        """
        return self.scene.history.canRedo()

    def getUserFriendlyFilename(self) -> str:
        """Get user friendly filename. Used in window title

        :return: just a base name of the file or `'New Graph'`
        :rtype: ``str``
        """
        name = os.path.basename(self.filename) if self.isFilenameSet() else "New Graph"
        return name + ("*" if self.isModified() else "")

    def fileNew(self):
        """Empty the scene (create new file)"""
        self.scene.clear()
        self.filename = None
        self.scene.history.clear()
        self.scene.history.storeInitialHistoryStamp()

    def fileLoad(self, filename:str):
        """Load serialized graph from JSON file

        :param filename: file to load
        :type filename: ``str``
        """
        QApplication.setOverrideCursor(Qt.WaitCursor)
        try:
            self.scene.loadFromFile(filename)
            self.filename = filename
            self.scene.history.clear()
            self.scene.history.storeInitialHistoryStamp()
            return True
        except FileNotFoundError as e:
            dumpException(e)
            QMessageBox.warning(self, "Error loading %s" % os.path.basename(filename), str(e).replace('[Errno 2]',''))
            return False
        except InvalidFile as e:
            dumpException(e)
            # QApplication.restoreOverrideCursor()
            QMessageBox.warning(self, "Error loading %s" % os.path.basename(filename), str(e))
            return False
        finally:
            QApplication.restoreOverrideCursor()


    def fileSave(self, filename:str=None):
        """Save serialized graph to JSON file. When called with empty parameter, we won't store/remember the filename

        :param filename: file to store the graph
        :type filename: ``str``
        """
        if filename is not None: self.filename = filename
        QApplication.setOverrideCursor(Qt.WaitCursor)
        self.scene.saveToFile(self.filename)
        QApplication.restoreOverrideCursor()
        return True


    def addNodes(self):
        """Testing method to create 3 `Nodes` with 3 `Edges` connecting them"""
        node1 = Node(self.scene, "My Awesome Node 1", inputs=[0,0,0], outputs=[1,5])
        node2 = Node(self.scene, "My Awesome Node 2", inputs=[3,3,3], outputs=[1])
        node3 = Node(self.scene, "My Awesome Node 3", inputs=[2,2,2], outputs=[1])
        node1.setPos(-350, -250)
        node2.setPos(-75, 0)
        node3.setPos(200, -200)

        edge1 = Edge(self.scene, node1.outputs[0], node2.inputs[0], edge_type=EDGE_TYPE_BEZIER)
        edge2 = Edge(self.scene, node2.outputs[0], node3.inputs[0], edge_type=EDGE_TYPE_BEZIER)
        edge3 = Edge(self.scene, node1.outputs[0], node3.inputs[2], edge_type=EDGE_TYPE_BEZIER)

        self.scene.history.storeInitialHistoryStamp()

    def addCustomNode(self):
        """Testing method to create a custom Node with custom content"""
        from nodeeditor.node_content_widget import QDMNodeContentWidget
        from nodeeditor.node_serializable import Serializable

        class NNodeContent(QLabel):  # , Serializable):
            def __init__(self, node, parent=None):
                super().__init__("FooBar")
                self.node = node
                self.setParent(parent)

        class NNode(Node):
            NodeContent_class = NNodeContent

        self.scene.setNodeClassSelector(lambda data: NNode)
        node = NNode(self.scene, "A Custom Node 1", inputs=[0, 1, 2])

        print("node content:", node.content)

    def addDebugContent(self):
        """Testing method to put random QGraphicsItems and elements into QGraphicsScene"""
        greenBrush = QBrush(Qt.green)
        outlinePen = QPen(Qt.black)
        outlinePen.setWidth(2)

        rect = self.grScene.addRect(-100, -100, 80, 100, outlinePen, greenBrush)
        rect.setFlag(QGraphicsItem.ItemIsMovable)

        text = self.grScene.addText("This is my Awesome text!", QFont("Ubuntu"))
        text.setFlag(QGraphicsItem.ItemIsSelectable)
        text.setFlag(QGraphicsItem.ItemIsMovable)
        text.setDefaultTextColor(QColor.fromRgbF(1.0, 1.0, 1.0))


        widget1 = QPushButton("Hello World")
        proxy1 = self.grScene.addWidget(widget1)
        proxy1.setFlag(QGraphicsItem.ItemIsMovable)
        proxy1.setPos(0, 30)


        widget2 = QTextEdit()
        proxy2 = self.grScene.addWidget(widget2)
        proxy2.setFlag(QGraphicsItem.ItemIsSelectable)
        proxy2.setPos(0, 60)


        line = self.grScene.addLine(-200, -200, 400, -100, outlinePen)
        line.setFlag(QGraphicsItem.ItemIsMovable)
        line.setFlag(QGraphicsItem.ItemIsSelectable)




from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#from component.OpenWeatherMap import SQlite_OpenWeatherMap

#from CoolProp.CoolProp import PropsSI

#from AHU import FreshAir
from AHU import FreshAir
#from AHU.air_humide import air_humide
from AHU.air_humide import air_humide

import sched, time

class CalcAirInputContent(QDMNodeContentWidget):
    def initUI(self):
        
        
        
        # self.fluid_lbl = QLabel("Type de fluide", self)
        # self.fluid=QComboBox(self)
        
        # self.fluid.addItem("ammonia")
        # self.fluid.addItem("water")
        # listeDesFluides=Fluid().listeDesFluides()
        
        # for i in listeDesFluides:
        #     self.fluid.addItem(i)        
        
        # self.fluid.view().setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)                   
        
        self.F_kgs_lbl=QLabel("Air humide (m3/h)", self) 
        self.V_flow_edit=QLineEdit("3000", self)
        
        self.T_lbl = QLabel("Température (°C)", self)  
        
        
          
          
        self.T_edit = QLineEdit("15", self)
        
        
        self.RH_lbl = QLabel("RH (%)", self)              
        self.RH_edit = QLineEdit("100", self)
        
        self.P_lbl = QLabel("Pression (bar)", self)                       
        self.P_edit = QLineEdit("1.01325", self)
        
        self.layout=QVBoxLayout()
        
        # self.layout.addWidget(self.fluid_lbl)
        # self.layout.addWidget(self.fluid)        
        self.layout.addWidget(self.F_kgs_lbl)
        self.layout.addWidget(self.V_flow_edit)        
        self.layout.addWidget(self.T_lbl)
        self.layout.addWidget(self.T_edit)
        
        self.layout.addWidget(self.RH_lbl)
        self.layout.addWidget(self.RH_edit)
        
        self.layout.addWidget(self.P_lbl)
        self.layout.addWidget(self.P_edit)
        
        
        
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        

    def serialize(self):
        res = super().serialize()
        res['value'] = self.T_edit.text()
        res2 = super().serialize()
        res2['P'] = self.P_edit.text()
        
        res3 = super().serialize()
        res3['RH'] = self.RH_edit.text()
        
        res4 = super().serialize()
        res4['V_flow'] = self.V_flow_edit.text()
        
        return res,res2,res3,res4

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        res2 = super().deserialize(data, hashmap)
        res3 = super().deserialize(data, hashmap)
        res4 = super().deserialize(data, hashmap)
       # print("res=",res,res2,res3,res4)
       # print("dataaaaaaaaaa=",data)
        try:
            
            T = data[0]["T"]
            P = data[1]['P']
            RH = data[2]['RH']
            V_flow = data[3]['V_flow']
            
          #  print("values=",value,P,RH,V_flow)
            
            self.T_edit.setText(T)
            self.P_edit.setText(P)
            self.RH_edit.setText(RH)
            self.V_flow_edit.setText(V_flow)
            
            return True & res & res2 & res3 & res4
        except Exception as e:
            dumpException(e)
        return res,res2,res3,res4


@register_node(OP_NODE_AIR_INPUT)
class CalcNode_AirInput(CalcNode):
    icon = "icons/in.png"
    op_code = OP_NODE_AIR_INPUT
    op_title = "Air Humide"
    content_label_objname = "calc_node_air_input"
    

    def __init__(self, scene):
        super().__init__(scene, inputs=[], outputs=[3])
        self.eval()

    def initInnerClasses(self):
        self.content = CalcAirInputContent(self)
        self.grNode = CalcGraphicsNode(self)
        self.grNode.height=300
        self.grNode.width=200
        self.content.T_edit.textChanged.connect(self.onInputChanged)
        self.content.P_edit.textChanged.connect(self.onInputChanged)
        self.content.V_flow_edit.textChanged.connect(self.onInputChanged)
        self.content.RH_edit.textChanged.connect(self.onInputChanged)

    def evalImplementation(self):
        
        s_T = float(self.content.T_edit.text())
        s_P = float(self.content.P_edit.text())  
        s_RH = float(self.content.RH_edit.text())
        s_F_kgs = float(self.content.V_flow_edit.text()) * air_humide.Air_rho_hum(s_T, s_RH, s_P*10**5)/3600 #kg/s
      #  print("masse vol=",air_humide.Air_rho_hum(s_T, s_RH, s_P*10**5))
        
         
        
        self.AN=FreshAir.Object()
        
        #lecture des données
        self.AN.F_m3h=s_F_kgs
        self.AN.T=s_T
      #  print("self.AN.T",self.AN.T)
        self.AN.RH=s_RH
        print("self.AN.RH",self.AN.RH)

        #Calculer les propriétés d'air neuf
        self.AN.calculate()
        
      #  print(self.AN.Inlet.propriete())
      #  print(self.AN.Outlet.propriete())
        
        
       
        
        
        
        self.value = [self.AN.w,s_F_kgs,s_P,self.AN.h]
     #   print("air humide",self.value)
        
        self.markDirty(False)
        self.markInvalid(False)

        self.markDescendantsInvalid(False)
        self.markDescendantsDirty()

        self.grNode.setToolTip("")

        self.evalChildren()
      #  print("données entrée = ",self.value)
        return self.value
   

       
            


        
    def propriete(self,Pro,I1,ValI1,I2,ValI2):
        result=PropsSI(Pro, I1, ValI1, I2, ValI2, self.fluid)
        return result


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from PyqtSimulator.calc_conf import *
from NodeEditor.nodeeditor.utils import dumpException


class QDMDragListbox(QListWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.initUI()

    def initUI(self):
        # init
        self.setIconSize(QSize(32, 32))
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setDragEnabled(True)

        self.addMyItems()


    def addMyItems(self):
        keys = list(CALC_NODES.keys())
        keys.sort()
        for key in keys:
            node = get_class_from_opcode(key)
            self.addMyItem(node.op_title, node.icon, node.op_code)


    def addMyItem(self, name, icon=None, op_code=0):
        item = QListWidgetItem(name, self) # can be (icon, text, parent, <int>type)
        pixmap = QPixmap(icon if icon is not None else ".")
        item.setIcon(QIcon(pixmap))
        item.setSizeHint(QSize(32, 32))

        item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsDragEnabled)

        # setup data
        item.setData(Qt.UserRole, pixmap)
        item.setData(Qt.UserRole + 1, op_code)


    def startDrag(self, *args, **kwargs):
        try:
            item = self.currentItem()
            op_code = item.data(Qt.UserRole + 1)

            pixmap = QPixmap(item.data(Qt.UserRole))


            itemData = QByteArray()
            dataStream = QDataStream(itemData, QIODevice.WriteOnly)
            dataStream << pixmap
            dataStream.writeInt(op_code)
            dataStream.writeQString(item.text())

            mimeData = QMimeData()
            mimeData.setData(LISTBOX_MIMETYPE, itemData)

            drag = QDrag(self)
            drag.setMimeData(mimeData)
            #drag.setHotSpot(QPoint(pixmap.width() / 2, pixmap.height() / 2))
            drag.setHotSpot(QPoint(int(pixmap.width() / 2), int(pixmap.height() / 2)))

            drag.setPixmap(pixmap)

            drag.exec_(Qt.MoveAction)

        except Exception as e: dumpException(e)


from AHU.air_humide import air_humide


from AHU.AirPort.AirPort import AirPort

class Object:
    def __init__(self):
        self.Inlet=AirPort() 
        self.Outlet=AirPort()
        self.P=0
        self.P_drop=0
        self.id=2
        # données air neuf
        self.T_in=0
        self.hi=0
        self.wi=12
        self.F=0
        self.F_dry=0
        
        #consigne
        self.To_target=20
        # calcul Outlet Coil
        self.ho=0
        self.Q_th=12
        self.RH_out=0
        
      
        
    def calculate(self):
        
          #connecteur  
        self.Outlet.P=self.Inlet.P-self.P_drop
      
        self.wi=self.Inlet.w
        self.P=self.Inlet.P
        self.hi=self.Inlet.h
        self.F=self.Inlet.F
        
        self.T_in=air_humide.Air_T_db(h=self.hi,w=self.wi)
        

        self.ho=air_humide.Air_h(T_db=self.To_target,w=self.wi)
          #  print("ho=",self.ho)
        self.F_dry=(self.F)/(1+(self.wi/1000))
        print("self.F_dry=",self.F_dry,"self.Inlet.P=",self.Inlet.P,"self.F=",self.F)
        self.Q_th=(self.ho-self.hi)*self.F_dry
          # print("self.Q_th=",self.Q_th)
        self.RH_out=air_humide.Air_RH(Pv_sat=air_humide.Air_Pv_sat(self.To_target),w=self.wi,P=self.Outlet.P) #parametrer la pression
          # print("self.RH_out=",self.RH_out)
        
       
            
            
            

        
        #connecteur   
      
          
        self.Outlet.w=self.Inlet.w
        
        self.Outlet.h=self.ho
        self.Outlet.F=self.Inlet.F #à corriger
        self.Outlet.F_dry=self.F_dry
#
       





from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#from AHU.air_humide import air_humide
from AHU.air_humide import air_humide
#from CoolProp.CoolProp import PropsSI


class CalcAirOutputContent(QDMNodeContentWidget):
    def initUI(self):
        
        
        self.HA_lbl = QLabel("", self)
        
        
        self.F_kgs_lbl = QLabel("", self)
        
       
        self.P_lbl = QLabel("", self)
        
        
        self.lbl = QLabel("", self)

        self.T_lbl = QLabel("", self)
        self.RH_lbl = QLabel("", self)
        
        
        
         
        self.layout=QVBoxLayout()

        
        self.layout.addWidget(self.HA_lbl)  
        
        
        self.layout.addWidget(self.F_kgs_lbl) 
      
        
        self.layout.addWidget(self.P_lbl)
        
  
        self.layout.addWidget(self.lbl)
        
      
        self.layout.addWidget(self.T_lbl)
        self.layout.addWidget(self.RH_lbl)
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)


@register_node(OP_NODE_AIR_OUTPUT)
class CalcNode_AirOutput(CalcNode):
    icon = "icons/out.png"
    op_code = OP_NODE_AIR_OUTPUT
    op_title = "AirOutput"
    content_label_objname = "calc_node_air_output"

    def __init__(self, scene):
        super().__init__(scene, inputs=[1], outputs=[])

    def initInnerClasses(self):
        self.content = CalcAirOutputContent(self)
        self.grNode = CalcGraphicsNode(self)
        self.grNode.height=200
      

    def evalImplementation(self):
        input_node = self.getInput(0)
        if not input_node:
            self.grNode.setToolTip("Input is not connected")
            self.markInvalid()
            return

        val = input_node.eval()
        print("les données d'entrée de output sont : ",val)

        if val is None:
            self.grNode.setToolTip("Input is NaN")
            self.markInvalid()
            return
        # try:
        #     self.T=PropsSI("T", "P", 100000*val[2], "H", val[3]*1000, val[0])-273.15
        # except:
        self.T=air_humide.Air_T_db(h=val[3],w=val[0])
        self.RH=air_humide.Air_RH(Pv_sat=air_humide.Air_Pv_sat(self.T),w=val[0],P=val[2]*100000)
        

        self.content.HA_lbl.setText("%.2f" % val[0]+"g/kg-as") #"%d" % 
        #     elif (self.Q<1 and self.Q>0):
        #         self.content.HA_lbl.setText(val[0]+" diphasique"+"%.2f" % self.Q) #"%d" % 
        #     else:
        #         self.content.HA_lbl.setText(val[0]+" liquide"+"%.2f" % self.Q) #"%d" % 
        #     print("Qualité=",self.Q)
            
        # else:
        #     self.content.HA_lbl.setText(val[0]+" supercritique") #"%d" % 
        
        
    
        
        
        self.content.F_kgs_lbl.setText("%.1f" % (val[1]*3600)+" kg/h")
        self.content.P_lbl.setText("%.3f" % val[2]+" bar")
        self.content.lbl.setText("%.3f" % val[3]+" kJ/kg-as")
        self.content.T_lbl.setText("%.1f" % self.T+ " °C")
        self.content.RH_lbl.setText("%.1f" % self.RH+ " %")
        
        self.markInvalid(False)
        self.markDirty(False)
        self.grNode.setToolTip("")

        return val



from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

from CoolProp.CoolProp import PropsSI


class CalcOutputContent(QDMNodeContentWidget):
    def initUI(self):
        
        
        self.fluid_lbl = QLabel("", self)
        
        
        self.F_kgs_lbl = QLabel("", self)
        self.NV_flow_lbl = QLabel("", self)
        self.V_flow_lbl = QLabel("", self)
        
       
        self.P_lbl = QLabel("", self)
        
        
        self.lbl = QLabel("", self)

        self.T_lbl = QLabel("", self)
        
        
        
         
        self.layout=QVBoxLayout()

        
        self.layout.addWidget(self.fluid_lbl)  
        
        
        self.layout.addWidget(self.F_kgs_lbl) 
        self.layout.addWidget(self.NV_flow_lbl) 
        self.layout.addWidget(self.V_flow_lbl)
      
        
        self.layout.addWidget(self.P_lbl)
        
  
        self.layout.addWidget(self.lbl)
        
      
        self.layout.addWidget(self.T_lbl)
        
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)


@register_node(OP_NODE_OUTPUT)
class CalcNode_Output(CalcNode):
    icon = "icons/out.png"
    op_code = OP_NODE_OUTPUT
    op_title = "Output"
    content_label_objname = "calc_node_output"

    def __init__(self, scene):
        super().__init__(scene, inputs=[1], outputs=[])

    def initInnerClasses(self):
        self.content = CalcOutputContent(self)
        self.grNode = CalcGraphicsNode(self)
        self.grNode.height=250
      

    def evalImplementation(self):
        input_node = self.getInput(0)
        if not input_node:
            self.grNode.setToolTip("Input is not connected")
            self.markInvalid()
            return

        val = input_node.eval()
       # print("les données d'entrée de output sont : ",val)

        if val is None:
            self.grNode.setToolTip("Input is NaN")
            self.markInvalid()
            return
        try:
            self.T=PropsSI("T", "P", 100000*val[2], "H", val[3]*1000, val[0])-273.15
        except:
           self.T=0-273.15
        
        if (100000*val[2])<PropsSI("Pcrit",val[0]): #comparer à la pression critique
            Hv=PropsSI("H", "P", 100000*val[2], "Q", 1, val[0])
           # print("Hv=",Hv)
            Hl=PropsSI("H", "P", 100000*val[2], "Q", 0, val[0])
          #  print("Hl=",Hl)
            self.Q=1-((Hv-val[3]*1000)/(Hv-Hl))
            
            if self.Q>=1:
                self.content.fluid_lbl.setText(val[0]+" vapeur"+"%.2f" % self.Q) #"%d" % 
            elif (self.Q<1 and self.Q>0):
                self.content.fluid_lbl.setText(val[0]+" diphasique"+"%.2f" % self.Q) #"%d" % 
            else:
                self.content.fluid_lbl.setText(val[0]+" liquide"+"%.2f" % self.Q) #"%d" % 
           # print("Qualité=",self.Q)
            
        else:
            self.content.fluid_lbl.setText(val[0]+" supercritique") #"%d" % 
            
         
        if (100000*val[2])<PropsSI("Pcrit",val[0]): #comparer à la pression critique
            if self.Q>1:
                V_flow=val[1]/PropsSI("D", "P", 100000*val[2], "T", (self.T+273.15), val[0])
            elif (self.Q<=1 and self.Q>=0):
                V_flow=self.Q*val[1]/PropsSI("D", "P", 100000*val[2], "Q", 1, val[0])+(1-self.Q)*val[1]/PropsSI("D", "P", 100000*val[2], "Q", 0, val[0])
            else:
               V_flow=val[1]/PropsSI("D", "P", 100000*val[2], "T", (self.T+273.15), val[0])        
        else:    
            V_flow=val[1]/PropsSI("D", "P", 100000*val[2], "T", (self.T+273.15), val[0])
        
        NV_flow=val[1]/PropsSI("D", "P", 100000*1.01325, "T", (15+273.15), val[0])
        
    
        
        
        self.content.NV_flow_lbl.setText("%.3f" % (NV_flow*3600)+" Nm3/h")
        self.content.V_flow_lbl.setText("%.3f" % (V_flow*3600)+" m3/h")
        
        self.content.F_kgs_lbl.setText("%.3f" % (val[1]*3600)+" kg/h")
        self.content.P_lbl.setText("%.3f" % val[2]+" bar")
        self.content.lbl.setText("%.3f" % val[3]+" kJ/kg-K")
        self.content.T_lbl.setText("%.1f" % self.T+ " °C")
        
        self.markInvalid(False)
        self.markDirty(False)
        self.grNode.setToolTip("")

        return val



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.read()

setup_requirements = [ ]

test_requirements = [ ]

import nodeeditor

setup(
    author="Pavel Křupala",
    author_email='pavel.krupala@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python Node Editor using PyQt5",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='nodeeditor',
    name='nodeeditor',
    #packages=find_packages(include=['_template']),
    packages=find_packages(include=['nodeeditor*'], exclude=['examples*', 'tests*']),
    package_data={'': ['qss/*']},
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://gitlab.com/pavel.krupala/pyqt-node-editor.git',
    version=nodeeditor.__version__,
    zip_safe=False,
)



import matplotlib.pyplot as plt
import numpy as np

def plot_streams_and_temperature_intervals(self, figsize=(12, 6), xticks_rotation=90):
    # Vérifier la présence des colonnes requises
    required_columns = ['StreamType', 'name', 'STi', 'STo', 'mCp', 'delta_H']
    for column in required_columns:
        if column not in self.stream_list.columns:
            raise ValueError(f"Colonne '{column}' manquante dans le DataFrame.")

    # Vérifier la non-nullité des colonnes nécessaires
    if self.stream_list[required_columns].isnull().values.any():
        raise ValueError("Le DataFrame contient des valeurs nulles dans les colonnes nécessaires.")

    # Extraire les colonnes nécessaires du DataFrame
    StreamType = self.stream_list['StreamType']
    names = self.stream_list['name']
    STi = self.stream_list['STi']
    STo = self.stream_list['STo']
    mCp = self.stream_list['mCp']
    delta_H = self.stream_list['delta_H']

    # Positions numériques sur l'axe x
    x_positions = np.arange(len(names))

    # Créer une nouvelle figure
    plt.figure(figsize=figsize)

    # Tracer les flux avec l'échelle de température décalée et la couleur en fonction du StreamType
    for i, x in enumerate(x_positions):
        if StreamType.iloc[i] == 'HS':  # Hot Stream
            plt.plot([x, x], [STi.iloc[i], STo.iloc[i]], color='red')
            plt.annotate('', xy=(x, STo.iloc[i]), xytext=(x, STo.iloc[i] + 1),
                         arrowprops=dict(arrowstyle='->', color='red', lw=1), annotation_clip=False)
        else:  # Cold Stream
            plt.plot([x, x], [STi.iloc[i], STo.iloc[i]], color='blue')
            plt.annotate('', xy=(x, STo.iloc[i]), xytext=(x, STo.iloc[i] - 1),
                         arrowprops=dict(arrowstyle='->', color='blue', lw=1), annotation_clip=False)

        # Calculer la position verticale moyenne pour l'annotation
        mid_temp = (STi.iloc[i] + STo.iloc[i]) / 2
        plt.text(x, mid_temp, f'{round(delta_H.iloc[i])} kW',
                 verticalalignment='center', horizontalalignment='center',
                 fontsize=8, color='black')

    # Configurer les ticks et labels de l'axe y
    y_ticks = sorted(set(STi) | set(STo))
    plt.yticks(y_ticks)

    # Ajouter des lignes horizontales en pointillé pour chaque valeur de température
    for temp in y_ticks:
        plt.axhline(y=temp, linestyle='--', color='gray', alpha=0.7)

    # Définir les labels des axes et le titre
    plt.xlabel('Stream name')
    plt.ylabel('Shifted temperature (°C)')
    plt.title('Streams and temperature intervals')

    # Associer les positions numériques aux labels (noms + mCp)
    plt.xticks(
        x_positions,
        [f'{name}\n($mCp={mCp_val:.2f}$)' for name, mCp_val in zip(names, mCp)],
        fontsize=8,
        rotation=xticks_rotation
    )

    # Ajuster la marge en bas pour que les labels des xticks soient bien visibles
    plt.subplots_adjust(bottom=0.25)

    # Afficher le label de la température de pincement sur l'axe y
    plt.text(0, self.Pinch_Temperature,
             f'Pinch_Temperature = {self.Pinch_Temperature} °C',
             verticalalignment='bottom', horizontalalignment='left')

    # Afficher la grille
    plt.grid(True)

    # Afficher le graphique
    plt.show()

########################################################################################

def plot_GCC(self):

    # Tracer la courbe de composition avec les axes inversés
    plt.plot(self.GCC['cumulative_delta_H'], self.GCC['T_shifted'], marker='o', label='Courbe de Composition')

    # Ajouter des étiquettes et un titre au graphe
    plt.xlabel('Net heat flow (kW)')
    plt.ylabel('Shifted temperature (°C)')
    plt.title('Grand composite curve')

    # Ajouter la grille
    plt.grid(True)

    # Trouver l'index du maximum de T_shifted
    max_index = self.GCC['T_shifted'].idxmax()

    # Afficher la valeur de Heating_duty au niveau du maximum de T_shifted
    plt.text(self.GCC['cumulative_delta_H'][max_index], self.GCC['T_shifted'][max_index], f'Heating_duty = {self.Heating_duty} kW', verticalalignment='bottom', horizontalalignment='left')

    # Trouver l'index du minimum de T_shifted
    min_index = self.GCC['T_shifted'].idxmin()

    # Afficher la valeur de Cooling_duty au niveau du minimum de T_shifted
    plt.text(self.GCC['cumulative_delta_H'][min_index], self.GCC['T_shifted'][min_index], f'Cooling_duty = {self.Cooling_duty} kW', verticalalignment='bottom', horizontalalignment='right')


    # Afficher la valeur de Pinch_Temperature sur l'axe des ordonnées
    plt.text(0, self.Pinch_Temperature, f'Pinch_Temperature = {self.Pinch_Temperature} °C', verticalalignment='bottom', horizontalalignment='left')


    # Afficher le graphe
    plt.show()

#########################################################################################


def plot_composites_curves(self):
        # Assurer que les deux dataframes ont la même longueur
    length = max(len(self.df_hcc), len(self.df_ccc))
    self.df_hcc = self.df_hcc.reindex(range(length)).fillna(method='ffill')
    self.df_ccc = self.df_ccc.reindex(range(length)).fillna(method='ffill')

    # Tracer les données originales
    plt.plot(self.df_hcc['Q'], self.df_hcc['T'], label='courbe composite du flux chaud', marker='o')
    plt.plot(self.df_ccc['Q'], self.df_ccc['T'], label='courbe composite du flux froid', marker='o')

    # Déterminer les limites pour la région à remplir
    q_min_fill = max(self.df_hcc['Q'].min(), self.df_ccc['Q'].min())
    q_max_fill = min(self.df_hcc['Q'].max(), self.df_ccc['Q'].max())

    # Créer une séquence linéaire pour Q dans la zone à remplir
    q_fill = np.linspace(q_min_fill, q_max_fill, 100)

    # Créer des masques pour exclure certaines parties de la zone remplie
    mask_before_ccc = q_fill < self.df_hcc['Q'].min()
    mask_after_ccf = q_fill > self.df_ccc['Q'].max()

    # Appliquer les masques pour exclure les parties indésirables
    q_fill_masked = q_fill[~(mask_before_ccc | mask_after_ccf)]
    t_ccc_fill = np.interp(q_fill_masked, self.df_hcc['Q'], self.df_hcc['T'])
    t_ccf_fill = np.interp(q_fill_masked, self.df_ccc['Q'], self.df_ccc['T'])

    # Tracer la zone remplie avec rotation
    plt.fill_between(q_fill_masked, t_ccc_fill, t_ccf_fill, color='green', alpha=0.3)


    # Ajouter des lignes pointillées
    for index, row in self.df_hcc.iterrows():
        plt.axvline(x=row['Q'], color='gray', linestyle='--')
        plt.axhline(y=row['T'], color='gray', linestyle='--')

    for index, row in self.df_ccc.iterrows():
        plt.axvline(x=row['Q'], color='gray', linestyle='--')
        plt.axhline(y=row['T'], color='gray', linestyle='--')

    # Ajouter des étiquettes et un titre
    plt.xlabel('Heat flow (kW)')
    plt.ylabel('Shifted temperature (°C)')
    plt.title('Shifted hot and cold composite curves')
    plt.legend()  # Ajouter la légende

    # Afficher le graphique
    plt.show()




from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#from component.OpenWeatherMap import SQlite_OpenWeatherMap

#from CoolProp.CoolProp import PropsSI

from AHU import FreshAir
#from OpenWeatherMap import OpenWeatherMap_call
from OpenWeatherMap import OpenWeatherMap_call_location
from AHU.air_humide import air_humide

from pyqtgraph import PlotWidget, plot
import pyqtgraph as pg
import numpy as np

import sched, time

hour=[]
temperature=[]


        
class CalcRandomMeteoContent(QDMNodeContentWidget):
    def initUI(self):
        
        
        
###################################Météo#######################################
        
        self.graphWidget = pg.PlotWidget()
        self.lat_lbl = QLabel("latitude", self)  
        self.lat_edit = QLineEdit("48.861341", self)
        self.lon_lbl = QLabel("longitude (°C)", self)  
        self.lon_edit = QLineEdit("2.337405", self)


###########################################################################                 
        
        self.V_flow_lbl=QLabel("Air humide (m3/h)", self) 
        self.V_flow_edit=QLineEdit("3000", self)
        
        self.T_lbl = QLabel("Température (°C)", self)  
        self.T_edit = QLineEdit("20", self)
        
        self.RH_lbl = QLabel("RH (%)", self)              
        self.RH_edit = QLineEdit("60", self)
        
        import threading
        import random
        
        import pandas as pd
        
        
        
        

        
        def printit():
            threading.Timer(10.0, printit).start()
            
           
            
            #API_T_RH=OpenWeatherMap_call.API_call()
           # print(self.lat_edit.text(),self.lon_edit.text())
            API_T_RH=OpenWeatherMap_call_location.API_call_location(self.lat_edit.text(),self.lon_edit.text())
            
            T=55*random.random()-15
            T=API_T_RH[0]
            RH=100*random.random()
            RH=API_T_RH[1]
            print("après 10 seconde T=",T)
         
            self.T_edit.setText("%f" % T)
            self.RH_edit.setText("%f" % RH)
            
            
                    ##############################################################
            
            temperature.append(float(self.T_edit.text()))  # [30,32,34,32,33,31,29,32,35,45]
          #  print(temperature)
            
            TimeIndex=len(temperature)
            hour.append(TimeIndex) #[1,2,3,4,5,6,7,8,9,10]
           # print(hour)

            # plot data: x, y values
            self.graphWidget.plot(hour, temperature)
        ######################################################
                                             
        printit()
        
            #hing to run
        


        
        
        
        self.P_lbl = QLabel("Pression (bar)", self)                       
        self.P_edit = QLineEdit("1.01325", self)
        
        
        
        
        self.layout=QVBoxLayout()
        
        self.layout.addWidget(self.lat_lbl)
        self.layout.addWidget(self.lat_edit)
        self.layout.addWidget(self.lon_lbl)
        self.layout.addWidget(self.lon_edit)
        
        # self.layout.addWidget(self.fluid_lbl)
        self.layout.addWidget(self.graphWidget)        
        self.layout.addWidget(self.V_flow_lbl)
        self.layout.addWidget(self.V_flow_edit)        
        self.layout.addWidget(self.T_lbl)
        self.layout.addWidget(self.T_edit)
        
        self.layout.addWidget(self.RH_lbl)
        self.layout.addWidget(self.RH_edit)
        
        self.layout.addWidget(self.P_lbl)
        self.layout.addWidget(self.P_edit)
        
        
        
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        

    def serialize(self):
        res = super().serialize()
        res['value'] = self.T_edit.text()
        res2 = super().serialize()
        res2['value2'] = self.P_edit.text()
        
        res3 = super().serialize()
        res3['value3'] = self.RH_edit.text()
        
        res4 = super().serialize()
        res4['value4'] = self.V_flow_edit.text()
        
        res5 = super().serialize()
        res5['lat'] = self.lat_edit.text()
        
        res6 = super().serialize()
        res6['lon'] = self.lon_edit.text()
        
        return res,res2,res3,res4,res5,res6

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        res2 = super().deserialize(data, hashmap)
        res3 = super().deserialize(data, hashmap)
        res4 = super().deserialize(data, hashmap)
        res5 = super().deserialize(data, hashmap)
        res6 = super().deserialize(data, hashmap)
        
       # print("res=",res,res2,res3,res4)
       # print("dataaaaaaaaaa=",data)
        try:
            
            value = data[0]["value"]
            value2 = data[1]['value2']
            value3 = data[2]['value3']
            value4 = data[3]['value4']
            value5 = data[4]['lat']
            value6 = data[5]['lon']
            
            
           # print("values=",value,value2,value3,value4)
            
            self.T_edit.setText(value)
            self.P_edit.setText(value2)
            self.RH_edit.setText(value3)
            self.V_flow_edit.setText(value4)
            self.lat_edit.setText(lat)
            self.lon_edit.setText(lon)
            
            
            return True & res & res2 & res3 & res4 & res5 & res6
        except Exception as e:
            dumpException(e)
        return res,res2,res3,res4


@register_node(OP_NODE_RANDOM_METEO)
class CalcNode_RandomMeteo(CalcNode):
    icon = "icons/in.png"
    op_code = OP_NODE_RANDOM_METEO
    op_title = "Météo"
    content_label_objname = "calc_node_random_meteo"
    

    def __init__(self, scene):
        super().__init__(scene, inputs=[], outputs=[3])
        self.eval()

    def initInnerClasses(self):
        self.content = CalcRandomMeteoContent(self)
        self.grNode = CalcGraphicsNode(self)
        self.grNode.height=450
        self.grNode.width=200
        self.content.T_edit.textChanged.connect(self.onInputChanged)
        self.content.P_edit.textChanged.connect(self.onInputChanged)
        self.content.V_flow_edit.textChanged.connect(self.onInputChanged)
        self.content.RH_edit.textChanged.connect(self.onInputChanged)
        self.content.lon_edit.textChanged.connect(self.onInputChanged)
        self.content.lat_edit.textChanged.connect(self.onInputChanged)

    def evalImplementation(self):
        u_T = self.content.T_edit.text()
        s_T = float(u_T)
        
        
        
        u_P = self.content.P_edit.text()
        s_P = float(u_P)
        
        s_RH = float(self.content.RH_edit.text())
        
        s_F_kgs = float(self.content.V_flow_edit.text())* air_humide.Air_rho_hum(s_T, s_RH, s_P*10**5)/3600
        
        
        
        
        
        

        
        # fluid=self.content.fluid.currentText()
        
        self.AN=AirNeuf.objet()
        
        #lecture des données
        self.AN.F=s_F_kgs
        self.AN.T=s_T
       # print("self.AN.T",self.AN.T)
        self.AN.RH=s_RH
      #  print("self.AN.RH",self.AN.RH)

        #Calculer les propriétés d'air neuf
        self.AN.calculate()
        
      #  print(self.AN.Inlet.propriete())
      #  print(self.AN.Outlet.propriete())
        
        
        self.h=self.AN.h
        
        
        
        self.value = [self.AN.w,s_F_kgs,s_P,self.h]
       # print("air humide",self.value)
        
        self.markDirty(False)
        self.markInvalid(False)

        self.markDescendantsInvalid(False)
        self.markDescendantsDirty()

        self.grNode.setToolTip("")

        self.evalChildren()
       # print("données entrée = ",self.value)
        return self.value
   

       
            


        
    def propriete(self,Pro,I1,ValI1,I2,ValI2):
        result=PropsSI(Pro, I1, ValI1, I2, ValI2, self.fluid)
        return result


"""
AirRecyclingAHU - Centrale de Traitement d'Air en mode recyclage.
"""

import logging
from typing import Dict, Optional
import pandas as pd

from AHU.air_humide import air_humide
from AHU.FreshAir.FreshAir import Object as FreshAir
from AHU.FreshAir.AirMix import Object as AirMix
from AHU.Coil.HeatingCoil import Object as HeatingCoil
from AHU.Coil.CoolingCoil_Expert import Object as CoolingCoil_Expert
from AHU.Humidification.Humidifier import Object as Humidifier
from AHU.Connect import Air_connect

logger = logging.getLogger(__name__)


class Object:
    """
    CTA en mode recyclage (melange air neuf + air recycle).

    Entrees:
        config (dict): configuration des composants
        data (pd.DataFrame): serie chronologique

    Sortie:
        df (pd.DataFrame): resultats de simulation (rempli apres calculate())
    """

    def __init__(self, config: Dict, data: pd.DataFrame):
        self.mode = "recycling"
        self.config = self._normalize_config(config)
        self.data = data
        self.df = None
        self.component_logs = {}
        logger.info("AirRecyclingAHU initialisee - %d lignes de donnees", len(data))
        logger.debug("Config: %s", self.config)

    @staticmethod
    def _normalize_config(config: Dict) -> Dict:
        defaults = {
            "recycling": False,
            "pre_heating_coil": False,
            "heating_coil": False,
            "cooling_coil": False,
            "humidifier": False,
            "humidifier_type": "adiabatique",
            "post_heating_coil": False
        }
        if config is None:
            raise ValueError("config est requis (dict).")

        normalized = defaults.copy()
        normalized.update(config)
        return normalized

    def _log_raw_air_point(self, key: str, air_obj, decimals: int = 2):
        if key not in self.component_logs:
            self.component_logs[key] = []

        self.component_logs[key].append({
            "T[°C]": round(air_obj.T, decimals),
            "RH[%]": round(air_obj.RH, decimals),
            "F_dry[kgas/s]": round(air_obj.F_dry, decimals),
            "h[kJ/kgda]": round(air_obj.h, decimals),
            "w[gH2O/kgda]": round(air_obj.w, decimals),
            "F_m3h[m3/h]": round(air_obj.F_dry / air_humide.Air_rho_hum(
                T_db=air_obj.T, RH=air_obj.RH) * 3600, decimals)
        })

    def _log_air_point(self, component_name: str, obj, Q_th: Optional[float] = None,
                       F_water: Optional[float] = None, decimals: int = 2):
        if component_name not in self.component_logs:
            self.component_logs[component_name] = []

        def extract_air_data(air_obj, label):
            return {
                f"{label}_T[°C]": round(air_obj.T, decimals),
                f"{label}_RH[%]": round(air_obj.RH, decimals),
                f"{label}_h[kJ/kgas]": round(air_obj.h, decimals),
                f"{label}_w[gH20/kgda]": round(air_obj.w, decimals),
                f"{label}_F[kg/s]": round(getattr(air_obj, "F", 0), decimals)
                if getattr(air_obj, "F", None) is not None else None,
                f"{label}_F_dry[kgas/s]": round(air_obj.F_dry, decimals),
                f"{label}F_m3h[m3/h]": round(air_obj.F_dry / air_humide.Air_rho_hum(
                    T_db=air_obj.T, RH=air_obj.RH) * 3600, decimals),
                f"{label}_P[Pa]": round(getattr(air_obj, "P", 0), decimals)
                if getattr(air_obj, "P", None) is not None else None,
                f"{label}_Pv_sat[Pa]": round(getattr(air_obj, "Pv_sat", 0), decimals)
                if getattr(air_obj, "Pv_sat", None) is not None else None
            }

        entry = {}

        if (hasattr(obj, "Inlet1") and hasattr(obj, "Outlet1") and
                hasattr(obj, "Inlet2") and hasattr(obj, "Outlet2")):
            entry.update(extract_air_data(obj.Inlet1, "Inlet1"))
            entry.update(extract_air_data(obj.Outlet1, "Outlet1"))
            entry.update(extract_air_data(obj.Inlet2, "Inlet2"))
            entry.update(extract_air_data(obj.Outlet2, "Outlet2"))

            if hasattr(obj, "heat_transfer1"):
                entry["Heat_recovery[kW]"] = round(obj.heat_transfer1, decimals)
            if hasattr(obj, "sensible_heat_transfer1"):
                entry["Sensible_heat1[kW]"] = round(obj.sensible_heat_transfer1, decimals)
            if hasattr(obj, "sensible_heat_transfer2"):
                entry["Sensible_heat2[kW]"] = round(obj.sensible_heat_transfer2, decimals)
            if hasattr(obj, "delta_mw1"):
                entry["Δmw1[gH2O/kgas]"] = round(obj.delta_mw1, decimals)
            if hasattr(obj, "delta_mw2"):
                entry["Δmw2[gH2O/kgas]"] = round(obj.delta_mw2, decimals)
            if hasattr(obj, "T_efficiency"):
                entry["T_efficiency[%]"] = round(obj.T_efficiency, decimals)
            if hasattr(obj, "h_efficiency"):
                entry["h_efficiency[%]"] = round(obj.h_efficiency, decimals)

        elif hasattr(obj, "Outlet"):
            entry.update(extract_air_data(obj.Outlet, "Outlet"))
            if Q_th is not None:
                entry["Q_th[kW]"] = round(Q_th, decimals)
            if F_water is not None:
                entry["F_water[kg/h]"] = round(F_water, decimals)

        self.component_logs[component_name].append(entry)

    def _compile_results(self) -> pd.DataFrame:
        merged_df = pd.DataFrame()

        for comp, logs in self.component_logs.items():
            if logs:
                df_comp = pd.DataFrame(logs)
                df_comp = df_comp.add_prefix(f"{comp}_")
                merged_df = pd.concat([merged_df, df_comp], axis=1)

        if self.data is not None and "Timestamp" in self.data.columns:
            merged_df.insert(0, "Timestamp", self.data["Timestamp"])

        return merged_df

    def calculate(self, progress_callback=None):
        """
        Lance la simulation en mode recyclage.
        Stocke les resultats dans self.df (pd.DataFrame).
        """
        if self.data is None:
            raise ValueError("Aucune donnee chargee. Fournissez data.")

        self.component_logs = {
            "FA": [], "RA": [], "AMX": [], "MXA": [],
            "PREHC": [], "HC": [], "CC": [], "HMD": [], "POSTHC": []
        }

        total = len(self.data)
        logger.info("Demarrage simulation RECYCLAGE - %d points", total)

        for index, row in self.data.iterrows():
            try:
                total_flow = row['Supply Air flow [m3/h]']
                recyc_pct = row['Mix Recycled Air [%]']
                T_consigne = row['Supply Air Set Point [T°C]']
                w_target = air_humide.Air_w(T_db=T_consigne,
                                            RH=row['Supply Air Set Point [HR %]'])

                FA = FreshAir()
                FA.T = row['Fresh Air [T°C]']
                FA.RH = row['Fresh Air [HR %]']
                FA.F_m3h = total_flow * (1 - recyc_pct / 100)
                FA.calculate()
                self._log_raw_air_point("FA", FA)
                logger.debug("Ligne %d - FA: T=%.1f RH=%.0f%%", index + 1, FA.T, FA.RH)

                if self.config["recycling"] and recyc_pct > 0:
                    RA = FreshAir()
                    RA.T = row['Recycled Air [T°C]']
                    RA.RH = row['Recycled Air [HR %]']
                    RA.F_m3h = total_flow * (recyc_pct / 100)
                    RA.calculate()
                    self._log_raw_air_point("RA", RA)

                    AMX = AirMix()
                    Air_connect(AMX.Inlet1, FA.Outlet)
                    Air_connect(AMX.Inlet2, RA.Outlet)
                    AMX.calculate()
                    self._log_air_point("AMX", AMX)

                    MXA = FreshAir()
                    Air_connect(MXA.Inlet, AMX.Outlet)
                    MXA.T = air_humide.Air_T_db(h=AMX.Outlet.h, w=AMX.Outlet.w)
                    MXA.RH = air_humide.Air_RH(h=AMX.Outlet.h, w=AMX.Outlet.w)
                    MXA.F_m3h = total_flow
                    MXA.calculate()
                else:
                    MXA = FreshAir()
                    MXA.T = FA.T
                    MXA.RH = FA.RH
                    MXA.F_m3h = total_flow
                    MXA.calculate()

                self._log_air_point("MXA", MXA)
                current_air = MXA

                if self.config["pre_heating_coil"]:
                    PREHC = HeatingCoil()
                    PREHC.To_target = row.get('Defrost Coil Set Point [T°C]', None)
                    Air_connect(PREHC.Inlet, current_air.Outlet)
                    PREHC.calculate()
                    self._log_air_point("PREHC", PREHC, Q_th=PREHC.Q_th)
                    current_air = PREHC
                    logger.debug("Ligne %d - PREHC: Q_th=%.2f kW", index + 1, PREHC.Q_th)

                if self.config["heating_coil"]:
                    HC = HeatingCoil()
                    HC.To_target = T_consigne
                    Air_connect(HC.Inlet, current_air.Outlet)
                    HC.calculate()
                    self._log_air_point("HC", HC, Q_th=HC.Q_th)
                    current_air = HC
                    logger.debug("Ligne %d - HC: Q_th=%.2f kW", index + 1, HC.Q_th)

                if self.config["cooling_coil"]:
                    CC = CoolingCoil_Expert()
                    CC.T_target = T_consigne
                    CC.w_target = w_target
                    Air_connect(CC.Inlet, current_air.Outlet)
                    CC.calculate()
                    self._log_air_point("CC", CC, Q_th=CC.Q_th)
                    current_air = CC
                    logger.debug("Ligne %d - CC: Q_th=%.2f kW", index + 1, CC.Q_th)

                if self.config["humidifier"]:
                    HMD = Humidifier()
                    HMD.wo_target = w_target
                    HMD.HumidType = self.config["humidifier_type"]
                    Air_connect(HMD.Inlet, current_air.Outlet)
                    HMD.calculate()
                    self._log_air_point("HMD", HMD, Q_th=HMD.Q_th, F_water=HMD.F_water)
                    current_air = HMD
                    logger.debug("Ligne %d - HMD: Q_th=%.2f kW, F_water=%.2f kg/h",
                                 index + 1, HMD.Q_th, HMD.F_water)

                if self.config["post_heating_coil"]:
                    POSTHC = HeatingCoil()
                    POSTHC.To_target = T_consigne
                    Air_connect(POSTHC.Inlet, current_air.Outlet)
                    POSTHC.calculate()
                    self._log_air_point("POSTHC", POSTHC, Q_th=POSTHC.Q_th)
                    current_air = POSTHC
                    logger.debug("Ligne %d - POSTHC: Q_th=%.2f kW", index + 1, POSTHC.Q_th)

                if progress_callback:
                    progress_callback(index + 1, total)

            except Exception as e:
                logger.error("Erreur ligne %d : %s", index + 1, e, exc_info=True)
                continue

        self.df = self._compile_results()
        logger.info("Simulation RECYCLAGE terminee - %d lignes, %d colonnes",
                     len(self.df), self.df.shape[1])





def decomposition_flux(self):
    # Ajouter des colonnes pour le nom du flux et la valeur de mCp à df_intervals
    self.df_intervals['StreamName'] = [[] for _ in range(len(self.df_intervals))]
    self.df_intervals['mCp'] = [[] for _ in range(len(self.df_intervals))]
    self.df_intervals['StreamType'] = [[] for _ in range(len(self.df_intervals))]  # Add this line

    #print('self.df_intervals===000=======',self.df_intervals)
    #print('self.stream_list==========',self.stream_list)
    # Parcourir chaque intervalle
    for idx, row in self.df_intervals.iterrows():

        # Parcourir chaque flux
        for i in range(self.rowCount):
            # Tester la condition spécifique (StreamType[i] == "CS")
            if (self.stream_list['StreamType'].iloc[i] == "CS") and (self.stream_list['STi'].iloc[i] < row['Tsup']) and (self.stream_list['STo'].iloc[i] > row['Tinf']):

                # Flux dans l'intervalle
                #self.df_intervals.at[idx, 'mCp'].append(-self.stream_list['mCp'][i])
                #self.df_intervals.at[idx, 'StreamName'].append(self.stream_list['name'][i])
                #self.df_intervals.at[idx, 'StreamType'].append(self.stream_list['StreamType'][i])  # Add this line
                self.df_intervals.at[idx, 'mCp'].append(-self.stream_list['mCp'].iloc[i])
                self.df_intervals.at[idx, 'StreamName'].append(self.stream_list['name'].iloc[i])
                self.df_intervals.at[idx, 'StreamType'].append(self.stream_list['StreamType'].iloc[i])





            # Tester la condition spécifique (StreamType[i] == "HS")
            #elif (self.stream_list['StreamType'][i] == "HS") and (self.stream_list['STi'][i] > row['Tinf']) and (self.stream_list['STo'][i] < row['Tsup']):
            elif (self.stream_list['StreamType'].iloc[i] == "HS") and (self.stream_list['STi'].iloc[i] > row['Tinf']) and (self.stream_list['STo'].iloc[i] < row['Tsup']):


                # Flux dans l'intervalle
                #self.df_intervals.at[idx, 'mCp'].append(self.stream_list['mCp'][i])
                self.df_intervals.at[idx, 'mCp'].append(self.stream_list['mCp'].iloc[i])
                #self.df_intervals.at[idx, 'StreamName'].append(self.stream_list['name'][i])
                self.df_intervals.at[idx, 'StreamName'].append(self.stream_list['name'].iloc[i])
                #self.df_intervals.at[idx, 'StreamType'].append(self.stream_list['StreamType'][i]) 
                self.df_intervals.at[idx, 'StreamType'].append(self.stream_list['StreamType'].iloc[i]) 


    #print('self.df_intervals====00=====',self.df_intervals)
    # Utiliser explode pour dupliquer les lignes pour chaque valeur de mCp
    self.df_intervals = self.df_intervals.explode(['StreamName', 'mCp', 'StreamType']).reset_index(drop=True)
    self.df_intervals = self.df_intervals.sort_values(by=['StreamName', 'Tsup']).reset_index(drop=True)
    self.df_intervals["delta_T"]=self.df_intervals['Tsup']-self.df_intervals['Tinf']
    self.df_intervals["delta_H"]=self.df_intervals["delta_T"]*self.df_intervals["mCp"]
    #print('self.df_intervals====0=====',self.df_intervals)




import json

euro_MWhcumac=5.0 #euro/MWhcumac
H1=[1,2,3,5,8,10,14,15,19,21,23,25,27,28,38,39,42,43,45,51,52,54,55,57,58,59,60,61,62,63,67,68,69,70,71,73,74,75,76,77,78,80,87,88,89,90,91,92,93,94,95,975]
H2=[4,7,9,12,16,17,18,22,24,26,29,31,32,33,35,36,37,40,41,44,46,47,48,49,50,53,56,64,65,72,79,81,82,84,85,86]
H3=[6,11,13,20,30,34,66,83,971,972,973,974,976]

#################################Transport#########################

def TRA_EQ_101(longueur_uti,nb_voyage_an,nb_uti):
    global euro_MWhcumac

    if longueur_uti=="UTIinf9":
        kWh_cumac=9300*(float(nb_voyage_an)*float(nb_uti))
    if longueur_uti=="UTIsup9":
        kWh_cumac=18500*(float(nb_voyage_an)*float(nb_uti))
    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac

    value = {
         "fiche_cee": "TRA_EQ_101",
         "titre": "Unité de transport intermodal pour le transport combiné rail-route",
        "euro_MWhcumac": euro_MWhcumac,
        "MWh_cumac": MWh_cumac,
        "euro": euro,
    }
 
    # Dictionary to JSON Object using dumps() method
    # Return JSON Object
    return json.dumps(value,ensure_ascii=False)

def TRA_EQ_107(type_bateau,bassin_navigation,nb_voyage_uti):
    global euro_MWhcumac

    if type_bateau=="Bateau DEK (1 000 t)":
        if bassin_navigation=="Seine":
            Ga=3800
        if bassin_navigation=="Rhône":
            Ga=3200 
        if bassin_navigation=="Nord Pas-de-Calais":
            Ga=3300 
        if bassin_navigation=="Rhin/Moselle":
            Ga=1200 
        if bassin_navigation=="Interbassin":
            Ga=2900
    if type_bateau=="Bateau RHK (1 350 t)":
        if bassin_navigation=="Seine":
            Ga=7900 
        if bassin_navigation=="Rhône":
            Ga=7500  
        if bassin_navigation=="Nord Pas-de-Calais":
            Ga=4000 
        if bassin_navigation=="Rhin/Moselle":
            Ga=2600 
        if bassin_navigation=="Interbassin":
            Ga=5500
    if type_bateau=="Bateau Grand Rhénan (2 500 t)":
        if bassin_navigation=="Seine":
            Ga=8500 
        if bassin_navigation=="Rhône":
            Ga=7800 
        if bassin_navigation=="Nord Pas-de-Calais":
            Ga=4700
        if bassin_navigation=="Rhin/Moselle":
            Ga=4100
        if bassin_navigation=="Interbassin":
            Ga=6300

    if type_bateau=="Bateau Convois (4 400 t)":
        if bassin_navigation=="Seine":
            Ga=9000
        if bassin_navigation=="Rhône":
            Ga=8500 
        if bassin_navigation=="Nord Pas-de-Calais":
            Ga=8300
        if bassin_navigation=="Rhin/Moselle":
            Ga=6500 
        if bassin_navigation=="Interbassin":
            Ga=8000

    
    kWh_cumac=Ga*float(nb_voyage_uti)
    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac

    value = {
         "fiche_cee": "TRA_EQ_107",
         "titre": "Unité de transport intermodal pour le transport combiné fluvial-route",
        "euro_MWhcumac": euro_MWhcumac,
        "MWh_cumac": MWh_cumac,
        "euro": euro,
    }
 
    # Dictionary to JSON Object using dumps() method
    # Return JSON Object
    return json.dumps(value,ensure_ascii=False)

def TRA_EQ_108(type_ligne,nb_voyage_ligne,distance_routiere_france,distance_feroviaire_france):
    global euro_MWhcumac

    if type_ligne=="Calais_Folkstone":
        kWh_cumac=6290*float(nb_voyage_ligne)
    if type_ligne=="Autre":
        kWh_cumac=(145*float(distance_routiere_france)-72*float(distance_feroviaire_france))*float(nb_voyage_ligne)
        if kWh_cumac<0:
            kWh_cumac=0

    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac

    value = {
         "fiche_cee": "TRA_EQ_108",
         "titre": "Wagon d'autoroute ferroviaire",
        "euro_MWhcumac": euro_MWhcumac,
        "MWh_cumac": MWh_cumac,
        "euro": euro,
    }
 
    # Dictionary to JSON Object using dumps() method
    # Return JSON Object
    return json.dumps(value,ensure_ascii=False)


# ########################################Industrie##################################
#fonctionnement : "1*8h" ,"2*8h" ,"8h_ArrWE" ,"3*8h_sansArrWE"
#Equipement_type : "pump","fan","air compressor","chiller"
#puissance_nominale : <=1000 kW , Puissance électrique nominale du moteur entraînant le système moto-régulé (en kW)
#Heat_Use : "chauffage de locaux","ECS","procédé industriel"
# puissance_nominale : Puissance nominal de l'échangeur ou au max la puissance électrique nominale du compresseur

def IND_UT_103(fonctionnement, Department, Heat_Use, puissance_nominale):

    global euro_MWhcumac
    global H1,H2,H3

    if Heat_Use=="chauffage de locaux" or Heat_Use=="ECS":

        if Department in H1:
            if fonctionnement=="1*8h" :
                kWhcumac_kW=6400
            if fonctionnement=="2*8h" :
                kWhcumac_kW=15900
            if fonctionnement=="3*8h_ArrWE" :
                kWhcumac_kW=19700
            if fonctionnement=="3*8h_sansArrWE":
                kWhcumac_kW=26700              

        if Department in H2:
            if fonctionnement=="1*8h" :
                kWhcumac_kW=6000
            if fonctionnement=="2*8h" :
                kWhcumac_kW=15000
            if fonctionnement=="3*8h_ArrWE" :
                kWhcumac_kW=18600
            if fonctionnement=="3*8h_sansArrWE":
                kWhcumac_kW=25200

        if Department in H3:
            if fonctionnement=="1*8h" :
                kWhcumac_kW=5000
            if fonctionnement=="2*8h" :
                kWhcumac_kW=12600
            if fonctionnement=="3*8h_ArrWE" :
                kWhcumac_kW=15600
            if fonctionnement=="3*8h_sansArrWE":
                kWhcumac_kW=21100

    if Heat_Use=="procédé industriel":
        if fonctionnement=="1*8h" :
            kWhcumac_kW=10300
        if fonctionnement=="2*8h" :
            kWhcumac_kW=25600
        if fonctionnement=="3*8h_ArrWE" :
            kWhcumac_kW=31800
        if fonctionnement=="3*8h_sansArrWE":
            kWhcumac_kW=43100

    kWh_cumac=kWhcumac_kW*puissance_nominale
    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac

    return "Système de récupération de chaleur sur un compresseur d’air : ",MWh_cumac, euro

def IND_UT_131(*Data):
    #fonctionnement,  Temperature,Geometry,S (Plan)
    #fonctionnement,Temperature, Geometry ,D,L (Cylindre)

    fonctionnement=Data[0]
    Temperature=Data[1]
    Geometery=Data[2]
    if Geometery=="cylindre":
        D=Data[3]
        L=Data[4]
        if D>=508:
            S=3.141592653589793*D/1000*L
        if D<508:
            S=None
    if Geometery=="plan":
        S=Data[3]
    
    if fonctionnement=="1*8h" :
        Coef=1.0
    if fonctionnement=="2*8h" :
        Coef=2.2
    if fonctionnement=="3*8h_ArrWE" :
        Coef=3.0
    if fonctionnement=="3*8h_sansArrWE":
        Coef=4.2
    
    if S is not None:
        print("equation plan ou grand diamètre")
        if -60 < Temperature <=0:
            kWhcumac_m2=80
        if 0 < Temperature <=40:
            kWhcumac_m2=0
        if 40 < Temperature <=100:
            kWhcumac_m2=190
        if 100 < Temperature <=300:
            kWhcumac_m2=490
        if 300 < Temperature <=600:
            kWhcumac_m2=1100
        kWh_cumac=kWhcumac_m2*Coef*S
    

    else:
        print("equation cylindre avec D<508 mm")
        if -60 < Temperature <=0:
            kWhcumac_m=53
        if 0 < Temperature <=40:
            kWhcumac_m=0
        if 40 < Temperature <=100:
            kWhcumac_m=110
        if 100 < Temperature <=300:
            kWhcumac_m=310
        if 300 < Temperature <=600:
            kWhcumac_m=850
        kWh_cumac=kWhcumac_m*Coef*L

    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac
    

    return "Isolation thermique des parois planes ou cylindriques sur des installations industrielles (France métropolitaine) : ",MWh_cumac, euro

def IND_UT_130(fonctionnement,puissance_nominale):
    global euro_MWhcumac
    if puissance_nominale<=20000:
        
        
        if fonctionnement=="1*8h" :
            kWhcumac_kW=340
        if fonctionnement=="2*8h" :
            kWhcumac_kW=740
        if fonctionnement=="3*8h_ArrWE" :
            kWhcumac_kW=1000
        if fonctionnement=="3*8h_sansArrWE"  :
            kWhcumac_kW=1400
        
       

       
            
        kWh_cumac=kWhcumac_kW*puissance_nominale
        MWh_cumac=kWh_cumac/1000
        euro=MWh_cumac*euro_MWhcumac

    else:
        MWh_cumac=0
        euro=0
    return "Condenseur sur les effluents gazeux d’une chaudière de production de vapeur : ",MWh_cumac, euro,

def IND_UT_134(fonctionnement, duree_contrat,puissance_nominale):
    global euro_MWhcumac
    if duree_contrat==1.0:
        F=1
    if duree_contrat==2.0:
        F=1.96
    if duree_contrat==3.0:
        F=2.89
    if duree_contrat==4.0:
        F=3.78
    if duree_contrat==5.0:
        F=4.63
    if duree_contrat>=6.0:
        F=5.45
    
    if fonctionnement=="1*8h" :
        kWh_cumac=29.4*1*puissance_nominale*F
    if fonctionnement=="2*8h" :
        kWh_cumac=29.4*2.2*puissance_nominale*F
    if fonctionnement=="3*8h_ArrWE" :
        kWh_cumac=29.4*3*puissance_nominale*F
    if fonctionnement=="3*8h_sansArrWE"  :
        kWh_cumac=29.4*4.2*puissance_nominale*F
    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac

    # données de Outlet
    value = {
         "fiche_cee": "IND_UT_134",
         "titre": "Système de mesurage d’indicateurs de performance énergétique",
        "euro_MWhcumac": euro_MWhcumac,
        "MWh_cumac": MWh_cumac,
        "euro": euro,
       
    }
 
    # Dictionary to JSON Object using dumps() method
    # Return JSON Object
    return json.dumps(value,ensure_ascii=False)

def IND_UT_135(fonctionnement, Department, Supply_Temperature, puissance_nominale):

    global euro_MWhcumac
    global H1,H2,H3

    if Department in H1:
        if 12 <= Supply_Temperature <15:
            kWhcumac_kW=7400
        if 15 <= Supply_Temperature <18:
            kWhcumac_kW=9900
        if 18 <= Supply_Temperature <=21:
            kWhcumac_kW=12300

    if Department in H2:
        if 12 <= Supply_Temperature <15:
            kWhcumac_kW=4900
        if 15 <= Supply_Temperature <18:
            kWhcumac_kW=8200
        if 18 <= Supply_Temperature <=21:
            kWhcumac_kW=11500

    if Department in H3:
        if 12 <= Supply_Temperature <15:
            kWhcumac_kW=3300
        if 15 <= Supply_Temperature <18:
            kWhcumac_kW=5800
        if 18 <= Supply_Temperature <=21:
            kWhcumac_kW=9000

    if fonctionnement=="1*8h" :
        Coef=1.0
    if fonctionnement=="2*8h" :
        Coef=2.2
    if fonctionnement=="3*8h_ArrWE" :
        Coef=3.0
    if fonctionnement=="3*8h_sansArrWE":
        Coef=4.2

    kWh_cumac=kWhcumac_kW*Coef*puissance_nominale
    MWh_cumac=kWh_cumac/1000
    euro=MWh_cumac*euro_MWhcumac


    return "Freecooling par eau de refroidissement en substitution d’un groupe froid : ",MWh_cumac, euro

def IND_UT_136(fonctionnement, Equipement_type,puissance_nominale):
    global euro_MWhcumac


    if puissance_nominale<=1000:
        if Equipement_type=="pump" or Equipement_type=="fan":
        
            if fonctionnement=="1*8h" :
                kWh_cumac=7800*puissance_nominale
            if fonctionnement=="2*8h" :
                kWh_cumac=17100*puissance_nominale
            if fonctionnement=="3*8h_ArrWE" :
                kWh_cumac=23300*puissance_nominale
            if fonctionnement=="3*8h_sansArrWE"  :
                kWh_cumac=32600*puissance_nominale
            MWh_cumac=kWh_cumac/1000
            euro=MWh_cumac*euro_MWhcumac

        if Equipement_type=="air compressor" or Equipement_type=="chiller":
        
            if fonctionnement=="1*8h" :
                kWhcumac_kW=4400
            if fonctionnement=="2*8h" :
                kWhcumac_kW=9800
            if fonctionnement=="3*8h_ArrWE" :
                kWhcumac_kW=13300
            if fonctionnement=="3*8h_sansArrWE":
                kWhcumac_kW=18600
            
            kWh_cumac=kWhcumac_kW*puissance_nominale
            MWh_cumac=kWh_cumac/1000
            euro=MWh_cumac*euro_MWhcumac

    else:
        MWh_cumac=0
        euro=0
    return "Systèmes moto-régulés : ",MWh_cumac, euro,


# ================================
# Registry & dispatcher
# ================================

DEPRECATED_FICHES = set()


def _normalize_fiche_code(fiche):
    if fiche is None:
        return None
    fiche = str(fiche).strip().upper()
    fiche = fiche.replace("_", "-")
    fiche = fiche.replace(" ", "")
    return fiche


def _as_dict_from_result(fiche_code, result):
    """Normalize result to a dict with kWh/MWh and euro."""
    if isinstance(result, str):
        try:
            data = json.loads(result)
        except json.JSONDecodeError:
            data = {"fiche_cee": fiche_code, "raw": result}
        return data

    if isinstance(result, (tuple, list)):
        # Expected format: (description, MWh_cumac, euro) or similar
        data = {
            "fiche_cee": fiche_code,
            "titre": result[0] if len(result) > 0 else None,
            "MWh_cumac": result[1] if len(result) > 1 else None,
            "euro": result[2] if len(result) > 2 else None,
            "euro_MWhcumac": euro_MWhcumac,
        }
        return data

    return {"fiche_cee": fiche_code, "raw": result}


def _ensure_kwh_cumac(data):
    if "kWh_cumac" not in data and "MWh_cumac" in data and data["MWh_cumac"] is not None:
        data["kWh_cumac"] = data["MWh_cumac"] * 1000
    return data


def list_fiches(include_deprecated=False):
    fiches = sorted(FICHE_REGISTRY.keys())
    if include_deprecated:
        return {"available": fiches, "deprecated": sorted(DEPRECATED_FICHES)}
    return fiches


def calcul_CEE(fiche, return_details=False, **kwargs):
    """
    Dispatcher générique de calcul CEE.

    Parameters
    ----------
    fiche : str
        Code fiche standardisée (ex: "IND-UT-103", "TRA-EQ-107").
    return_details : bool
        Si True, retourne un dict complet. Sinon retourne kWh_cumac.
    **kwargs : dict
        Paramètres spécifiques à la fiche.
    """
    fiche_code = _normalize_fiche_code(fiche)
    if fiche_code in DEPRECATED_FICHES:
        raise ValueError(f"La fiche {fiche_code} est supprimée par décret et n'est plus supportée.")

    if fiche_code not in FICHE_REGISTRY:
        raise ValueError(f"Fiche non supportée : {fiche_code}")

    result = FICHE_REGISTRY[fiche_code](**kwargs)
    data = _ensure_kwh_cumac(_as_dict_from_result(fiche_code, result))

    if return_details:
        return data
    return data.get("kWh_cumac")


def _wrap_TRA_EQ_101(longueur_uti, nb_voyage_an, nb_uti, **_):
    return TRA_EQ_101(longueur_uti, nb_voyage_an, nb_uti)


def _wrap_TRA_EQ_107(type_bateau, bassin_navigation, nb_voyage_uti, **_):
    return TRA_EQ_107(type_bateau, bassin_navigation, nb_voyage_uti)


def _wrap_TRA_EQ_108(type_ligne, nb_voyage_ligne, distance_routiere_france, distance_feroviaire_france, **_):
    return TRA_EQ_108(type_ligne, nb_voyage_ligne, distance_routiere_france, distance_feroviaire_france)


def _wrap_IND_UT_103(fonctionnement, Department, Heat_Use, puissance_nominale, **_):
    return IND_UT_103(fonctionnement, Department, Heat_Use, puissance_nominale)


def _wrap_IND_UT_130(fonctionnement, puissance_nominale, **_):
    return IND_UT_130(fonctionnement, puissance_nominale)


def _wrap_IND_UT_131(*, fonctionnement, Temperature, Geometry, S=None, D=None, L=None, **_):
    if Geometry == "plan":
        return IND_UT_131(fonctionnement, Temperature, Geometry, S)
    return IND_UT_131(fonctionnement, Temperature, Geometry, D, L)


def _wrap_IND_UT_134(fonctionnement, duree_contrat, puissance_nominale, **_):
    return IND_UT_134(fonctionnement, duree_contrat, puissance_nominale)


def _wrap_IND_UT_135(fonctionnement, Department, Supply_Temperature, puissance_nominale, **_):
    return IND_UT_135(fonctionnement, Department, Supply_Temperature, puissance_nominale)


def _wrap_IND_UT_136(fonctionnement, Equipement_type, puissance_nominale, **_):
    return IND_UT_136(fonctionnement, Equipement_type, puissance_nominale)


FICHE_REGISTRY = {
    "TRA-EQ-101": _wrap_TRA_EQ_101,
    "TRA-EQ-107": _wrap_TRA_EQ_107,
    "TRA-EQ-108": _wrap_TRA_EQ_108,
    "IND-UT-103": _wrap_IND_UT_103,
    "IND-UT-130": _wrap_IND_UT_130,
    "IND-UT-131": _wrap_IND_UT_131,
    "IND-UT-134": _wrap_IND_UT_134,
    "IND-UT-135": _wrap_IND_UT_135,
    "IND-UT-136": _wrap_IND_UT_136,
}





# =============================================================================
# Chiller Model (Evaporator + Compressor + Desuperheater + Condenser + Expansion_Valve)
# =============================================================================

import CoolProp.CoolProp as CP
import pandas as pd
from ThermodynamicCycles.Evaporator import Evaporator
from ThermodynamicCycles.Compressor import Compressor
from ThermodynamicCycles.Desuperheater import Desuperheater
from ThermodynamicCycles.Expansion_Valve import Expansion_Valve
from ThermodynamicCycles.Condenser import Condenser
from ThermodynamicCycles.Connect import Fluid_connect

from ThermodynamicCycles import Temperature_Entropy_Chart

class Object:
    def __init__(self, fluid='R134a', evap_params=None, comp_params=None, cond_params=None):
        # Initialize chiller components
        self.EVAP = Evaporator.Object()
        self.COMP = Compressor.Object()
        self.DESURCH = Desuperheater.Object()
        self.COND = Condenser.Object()
        self.DET = Expansion_Valve.Object()

        # Set fluid for all components
        self.fluid = fluid
        self.EVAP.fluid = self.fluid
        self.COMP.fluid = self.fluid
        self.DESURCH.fluid = self.fluid
        self.COND.fluid = self.fluid
        self.DET.fluid = self.fluid

        # Set parameters for each component
        self.set_evaporator_parameters(evap_params)
        self.set_compressor_parameters(comp_params)
        self.set_condenser_parameters(cond_params)

        self.points=[]

    def set_evaporator_parameters(self, params):
        if params:
            self.EVAP.Ti_degC= params.get('Ti_degC', 5)
            self.EVAP.surchauff = params.get('surchauff', 2)
            self.EVAP.Inlet.F = params.get('F', 1)
            

    def set_compressor_parameters(self, params):
        if params:
            self.COMP.Tcond_degC = params.get('Tcond_degC', 40)
            self.COMP.eta_is = params.get('eta_is', 0.8)
            self.COMP.Tdischarge_target = params.get('Tdischarge_target', 80)
            if 'Q_comp' in params:
                self.COMP.Q_comp = params['Q_comp']

    def set_condenser_parameters(self, params):
        if params:
            self.COND.subcooling = params.get('subcooling', 2)

    def calculate_cycle(self):
        # Calculation algorithm
        self.EVAP.Inlet.h = CP.PropsSI('H', 'P', 1 * 1e5, 'T', 40 + 273.15, self.fluid)
        self.EVAP.calculate() 
        Fluid_connect(self.COMP.Inlet, self.EVAP.Outlet)
        self.COMP.calculate()
        Fluid_connect(self.DESURCH.Inlet, self.COMP.Outlet)
        self.DESURCH.calculate()
        Fluid_connect(self.COND.Inlet, self.DESURCH.Outlet)
        self.COND.calculate()
        Fluid_connect(self.DET.Inlet, self.COND.Outlet)
        Fluid_connect(self.DET.Outlet, self.EVAP.Inlet)
        self.DET.calculate()
        Fluid_connect(self.EVAP.Inlet, self.DET.Outlet)
        self.EVAP.calculate() # Recalculate evaporator

        self.EER = self.EVAP.Q_evap / self.COMP.Q_comp
        self.Q_condTot = self.COND.Q_cond + self.DESURCH.Qdesurch
        self.COP = self.Q_condTot / self.COMP.Q_comp

        self.df = pd.DataFrame({
            
            'Chiller': [self.EER,self.COP,self.COMP.Q_comp/1000,self.COMP.Q_losses/1000,self.EVAP.Q_evap/1000,self.Q_condTot/1000]
        }, index=[ 'EER', 'COP','Q_comp (kW)','COMP.Q_losses (kW)','Q_evap (kW)', 'Q_condTot(kW)'])


        self.points = [
            {"T": self.EVAP.Tsv - 273.15, "S": self.EVAP.Ssv},
            {"T": self.EVAP.Outlet.T - 273.15, "S": self.EVAP.Outlet.S},
            {"T": self.COMP.Outlet.T - 273.15, "S": self.COMP.Outlet.S},
            {"T": self.DESURCH.Outlet.T - 273.15, "S": self.DESURCH.Outlet.S},
            {"T": self.COND.Tl_sat- 273.15, "S": self.COND.Sl_sat},
            {"T": self.COND.Outlet.T - 273.15, "S": self.COND.Outlet.S},
            {"T": self.DET.Outlet.T- 273.15, "S": self.DET.Outlet.S},
            {"T": self.EVAP.Tsv - 273.15, "S": self.EVAP.Ssv}
        ]
      



    def print_results(self):
        print(self.df)
        # Print Results
        print("COMPONENT DATAFRAMES:\n")
        print("Compressor:")
        print(self.COMP.df)
        print("\nEvaporator:")
        print(self.EVAP.df)
        print("\nDesuperheater:")
        print(self.DESURCH.df)
        print("\nCondenser:")
        print(self.COND.df)
        print("\nExpansion Valve:")
        print(self.DET.df)



    def plot_TS_diagram(self,figsize=(10, 6)):
        # Create a Temperature-Entropy chart object
        chart = Temperature_Entropy_Chart.Object(self.fluid)


        # Add points to the chart
        chart.add_points(self.points)

        # Display the chart
        chart.show(draw_arrows=True,figsize=figsize)





from AHU.air_humide import air_humide
from AHU.air_humide import air_humide_NB
from AHU.AirPort.AirPort import AirPort


class Object:
    def __init__(self):
        self.Inlet=AirPort() 
        self.Outlet=AirPort()
        
        #parameter :
        self.P_drop=0
        
        '''Point de consignes'''
        self.w_target=8 
        self.T_target=0
        
        '''Température point de rosée de l air au contact de la Coil froide '''
        self.T_sat=7  
        '''Températures inlet et outlet'''
        self.T_inlet=0
        self.T_Outlet=0
        
        '''Déshumidification Type'''
        self.Déshutype="Droite_sat"
                
        """facteur de bypass"""
        self.FB=0.2
        self.w_sat=0
        self.h_sat=0
        # calcul Outlet Coil
       
        self.Q_th=12      
        self.Eff=0.8 #efficacité de la Coil froide
        
        self.F=0
        self.F_dry=0 #Débit d'air sec
        
    def calculate(self):
        #print("self.Inlet.P=",self.Inlet.P,"air_humide.Air_Pv_sat(self.T_sat)=",air_humide.Air_Pv_sat(self.T_sat),"self.T_sat=",self.T_sat)
        self.w_sat=air_humide.Air_w(T_db=self.T_sat, RH=100, P=self.Inlet.P)
        self.h_sat=air_humide.Air_h(T_db=self.T_sat,w=self.w_sat)
        self.Outlet.w=self.w_target
        self.T_inlet=air_humide_NB.Air3_Tdb(self.Inlet.w/1000, self.Inlet.P, self.Inlet.h)
        self.F=self.Inlet.F
        #print("self.w_sat=",self.w_sat,"self.w_target=",self.w_target)
        
        '''Refroidissement jusqu'à la témpérature de consigne'''
        
        
        # if self.T_inlet>=self.T_target and self.Inlet.w>=self.w_sat:
        #     self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
        #     self.FB=1-self.Eff
        #     self.Outlet.h=self.h_sat+self.Eff*(self.Inlet.h-self.h_sat)
        #     self.Outlet.w=self.w_sat+self.Eff*(self.Inlet.w-self.w_sat)
            
        #if self.T_inlet>=self.T_target and self.Inlet.w<=self.w_sat :
        if self.T_inlet>=self.T_target :
            self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
            self.FB=1-self.Eff
            self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
            self.Outlet.w=self.Inlet.w
                        
            ''' sans aucune action de la Coil froide (Pas de traitement à faire) '''
        else:
            self.Outlet.h=self.Inlet.h
            self.Outlet.w=self.Inlet.w
            self.Eff=0
            self.FB=1-self.Eff
    
    
        self.Outlet.P=self.Inlet.P-self.P_drop
        self.F_dry=(self.Inlet.F)/(1+(self.Inlet.w/1000))
        #print("self.F_dry=",self.F_dry)
        self.Q_th=(self.Outlet.h-self.Inlet.h)*self.F_dry       
        #self.Outlet.F=self.F_dry*air_humide_NB.Air3_Vs(self.Outlet.w/1000,self.Outlet.P,self.Outlet.h) #[kg air sec/s] * [m3/kg air sec] =[m3/s]
        self.Outlet.F=self.F_dry*(1+(self.Outlet.w/1000))
        #print("self.Inlet.F=",self.Inlet.F)
        #print("self.Outlet.F=",self.Outlet.F)
        self.Outlet.F_dry=self.F_dry
        
 
           
            
            


